from __future__ import annotations

import os
from unittest.mock import MagicMock

import pytest
from click.testing import CliRunner
from vunnel import cli, provider, providers, result
from vunnel.providers import nvd


def test_list() -> None:
    runner = CliRunner()
    res = runner.invoke(cli.cli, ["list"])
    assert res.exit_code == 0

    expected_names = providers.names()
    assert len(expected_names) > 0

    expected_output = "\n".join(sorted(expected_names))
    assert expected_output.strip() == res.output.strip()


def test_status(helpers, tmpdir, monkeypatch) -> None:
    data_path = helpers.local_dir("test-fixtures/data-1")

    envs = {
        "NVD_API_KEY": "secret",
        "GITHUB_TOKEN": "secret",
    }
    monkeypatch.setattr(os, "environ", envs)

    config = tmpdir.join("vunnel.yaml")
    config.write(f"root: {data_path}")

    runner = CliRunner()
    res = runner.invoke(cli.cli, ["-c", str(config), "status"])
    assert res.exit_code == 0

    expected_output = f"""{data_path}
└── wolfi
       results: 56
       from:    2023-01-17 14:58:13
"""
    assert expected_output.strip() == res.output.strip()


def test_status_json(helpers, tmpdir, monkeypatch) -> None:
    import json

    data_path = helpers.local_dir("test-fixtures/data-1")

    envs = {
        "NVD_API_KEY": "secret",
        "GITHUB_TOKEN": "secret",
    }
    monkeypatch.setattr(os, "environ", envs)

    config = tmpdir.join("vunnel.yaml")
    config.write(f"root: {data_path}")

    runner = CliRunner()
    res = runner.invoke(cli.cli, ["-c", str(config), "status", "--json"])
    assert res.exit_code == 0

    # Parse and verify JSON output
    output_data = json.loads(res.output)

    expected_output = {
        "root": data_path,
        "providers": [
            {
                "name": "wolfi",
                "count": 56,
                "date": "2023-01-17 14:58:13",
                "error": None,
                "enabled": True
            }
        ]
    }

    assert output_data == expected_output

def test_run(mocker, monkeypatch) -> None:
    populate_mock = MagicMock()
    create_mock = MagicMock(return_value=populate_mock)
    mocker.patch.object(providers, "create", create_mock)

    envs = {"NVD_API_KEY": "secret"}
    monkeypatch.setattr(os, "environ", envs)

    runner = CliRunner()
    res = runner.invoke(cli.cli, ["-c", "-", "run", "nvd"])
    assert res.exit_code == 0
    assert populate_mock.run.call_count == 1
    assert create_mock.call_args_list == [
        mocker.call(
            "nvd",
            "./data",
            # note: this is the default config
            config=nvd.Config(
                runtime=provider.RuntimeConfig(
                    on_error=provider.OnErrorConfig(
                        action=provider.OnErrorAction.FAIL,
                        retry_count=3,
                        retry_delay=5,
                        input=provider.InputStatePolicy.KEEP,
                        results=provider.InputStatePolicy.KEEP,
                    ),
                    existing_input=provider.InputStatePolicy.KEEP,
                    existing_results=provider.InputStatePolicy.KEEP,
                    result_store=result.StoreStrategy.SQLITE,
                    import_results_host="",
                    import_results_path="providers/{provider_name}/listing.json",
                    import_results_enabled=False,
                ),
                request_timeout=125,
                api_key="secret",
            ),
        ),
    ]


@pytest.mark.parametrize(
    ("args", "clear", "clear_input", "clear_results"),
    (
        (["wolfi"], 1, 0, 0),
        (["wolfi", "-i"], 0, 1, 0),
        (["wolfi", "-r"], 0, 0, 1),
    ),
)
def test_clear(mocker, monkeypatch, args, clear, clear_input, clear_results) -> None:
    workspace_mock = MagicMock()
    create_mock = MagicMock(return_value=MagicMock(workspace=workspace_mock))
    mocker.patch.object(providers, "create", create_mock)

    runner = CliRunner()
    res = runner.invoke(cli.cli, ["clear", *args])
    assert res.exit_code == 0
    assert workspace_mock.clear.call_count == clear
    assert workspace_mock.clear_input.call_count == clear_input
    assert workspace_mock.clear_results.call_count == clear_results


def test_config(monkeypatch) -> None:
    envs = {
        "NVD_API_KEY": "secret",
        "GITHUB_TOKEN": "secret",
    }
    monkeypatch.setattr(os, "environ", envs)

    runner = CliRunner()
    res = runner.invoke(cli.cli, ["-c", "-", "config"])
    assert res.exit_code == 0
    expected_output = """
log:
  level: INFO
  show_level: true
  show_timestamp: false
  slim: false
providers:
  alma:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  alpine:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  amazon:
    max_allowed_alas_http_403: 25
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
    security_advisories:
      '2': https://alas.aws.amazon.com/AL2/alas.rss
      '2022': https://alas.aws.amazon.com/AL2022/alas.rss
      '2023': https://alas.aws.amazon.com/AL2023/alas.rss
  bitnami:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  chainguard:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  common:
    import_results:
      enabled: false
      host: ''
      path: providers/{provider_name}/listing.json
      skip_newer_archive_check: false
  debian:
    releases:
      bookworm: '12'
      bullseye: '11'
      buster: '10'
      jessie: '8'
      sid: unstable
      stretch: '9'
      trixie: '13'
      wheezy: '7'
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  echo:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  epss:
    dataset: current
    request_timeout: 125
    runtime:
      existing_input: delete
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
    url_template: https://epss.cyentia.com/epss_scores-{}.csv.gz
  github:
    api_url: https://api.github.com/graphql
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
    token: secret
  kev:
    request_timeout: 125
    runtime:
      existing_input: delete
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
    url: https://www.cisa.gov/sites/default/files/feeds/known_exploited_vulnerabilities.json
  mariner:
    allow_versions:
      - '1.0'
      - '2.0'
      - '3.0'
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  minimos:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  nvd:
    api_key: secret
    overrides_enabled: false
    overrides_url: https://github.com/anchore/nvd-data-overrides/archive/refs/heads/main.tar.gz
    request_retry_count: 10
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: keep
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  oracle:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  rhel:
    full_sync_interval: 2
    ignore_hydra_errors: false
    parallelism: 4
    request_timeout: 125
    rhsa_source: CSAF
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
    skip_namespaces:
      - rhel:3
      - rhel:4
  rocky:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  sles:
    allow_versions:
      - '11'
      - '12'
      - '15'
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  ubuntu:
    additional_versions: {}
    enable_rev_history: true
    git_branch: master
    git_url: git://git.launchpad.net/ubuntu-cve-tracker
    parallelism: 8
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: keep
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
  wolfi:
    request_timeout: 125
    runtime:
      existing_input: keep
      existing_results: delete-before-write
      import_results_enabled: false
      import_results_host: ''
      import_results_path: providers/{provider_name}/listing.json
      on_error:
        action: fail
        input: keep
        results: keep
        retry_count: 3
        retry_delay: 5
      result_store: sqlite
      skip_download: false
      skip_newer_archive_check: false
root: ./data
"""
    assert expected_output.strip() in res.output



from __future__ import annotations

import logging
import os
from typing import TYPE_CHECKING, Any

import orjson

if TYPE_CHECKING:
    from collections.abc import Generator

    from vunnel.workspace import Workspace

from .client import Client


class Parser:
    def __init__(self, ws: Workspace, logger: logging.Logger | None = None, skip_download: bool = False):
        self.workspace = ws
        if not logger:
            logger = logging.getLogger(self.__class__.__name__)
        self.logger = logger
        self.client = Client(
            download_path=os.path.join(self.workspace.input_path, "osv"),
            logger=self.logger,
            skip_download=skip_download,
        )
        self.urls = self.client.urls

    def _load(self) -> Generator[dict[str, Any], None, None]:
        for path in self.client.get():
            with open(path, encoding="utf-8") as f:
                yield orjson.loads(f.read())

    def _normalize(self, vuln_entry: dict[str, Any]) -> tuple[str, str, dict[str, Any]]:
        self.logger.trace("normalizing vulnerability data")  # type: ignore[attr-defined]
        vuln_id = vuln_entry["id"]
        vuln_schema = vuln_entry.get("schema_version", "1.7.0")
        return vuln_id, vuln_schema, vuln_entry

    def get(self) -> Generator[tuple[str, str, dict[str, Any]], None, None]:
        for vuln_entry in self._load():
            yield self._normalize(vuln_entry)



from __future__ import annotations

import os
from unittest.mock import patch

import pytest

from vunnel import result, workspace
from vunnel.providers.rhel import Config, Provider
from vunnel.providers.rhel.parser import Advisory, AffectedRelease, FixedIn, Parser
from vunnel.providers.rhel.rhsa_provider import OVALRHSAProvider


class TestParser:
    # flake8: noqa: E501
    @pytest.fixture
    def mock_cve(self):
        return {
            "threat_severity": "Low",
            "public_date": "2019-03-21T00:00:00Z",
            "bugzilla": {
                "description": "CVE-2019-9755 ntfs-3g: heap-based buffer overflow leads to local root privilege escalation",
                "id": "1691624",
                "url": "https://bugzilla.redhat.com/show_bug.cgi?id=1691624",
            },
            "cvss3": {
                "cvss3_base_score": "3.3",
                "cvss3_scoring_vector": "CVSS:3.0/AV:L/AC:L/PR:N/UI:R/S:U/C:N/I:N/A:L",
                "status": "verified",
            },
            "cwe": "CWE-122",
            "details": [
                "An integer underflow issue exists in ntfs-3g 2017.3.23. A local attacker could potentially exploit this by running /bin/ntfs-3g with specially crafted arguments from a specially crafted directory to cause a heap buffer overflow, resulting in a crash or the ability to execute arbitrary code. In installations where /bin/ntfs-3g is a setuid-root binary, this could lead to a local escalation of privileges."
            ],
            "statement": "This flaw has a lower impact on Red Hat Enterprise Linux because the ntfs-3g tool is run in a supermin appliance, which is similar to a virtual machine instantiated on the fly, and it does not have the SUID bit set. Thus an attacker is very limited on what he can do to the vulnerable system.",
            "affected_release": [
                {
                    "product_name": "Advanced Virtualization for RHEL 8.1.0",
                    "release_date": "2019-11-06T00:00:00Z",
                    "advisory": "RHBA-2019:3723",
                    "cpe": "cpe:/a:redhat:advanced_virtualization:8.1::el8",
                },
                {
                    "product_name": "Red Hat Enterprise Linux 7",
                    "release_date": "2019-08-06T00:00:00Z",
                    "advisory": "RHSA-2019:2308",
                    "cpe": "cpe:/o:redhat:enterprise_linux:7",
                    "package": "libguestfs-winsupport-7.2-3.el7",
                },
                {
                    "product_name": "Red Hat Enterprise Linux 8",
                    "release_date": "2019-11-05T00:00:00Z",
                    "advisory": "RHSA-2019:3345",
                    "cpe": "cpe:/a:redhat:enterprise_linux:8",
                    "package": "virt:rhel-8010020190916153839.cdc1202b",
                },
            ],
            "package_state": [
                {
                    "product_name": "Red Hat Enterprise Linux 8 Advanced Virtualization",
                    "fix_state": "Affected",
                    "package_name": "virt:8.0.0/libguestfs-winsupport",
                    "cpe": "cpe:/a:redhat:advanced_virtualization:8::el8",
                },
                {
                    "product_name": "Red Hat Enterprise Linux 6",
                    "fix_state": "Will not fix",
                    "package_name": "libguestfs-winsupport",
                    "cpe": "cpe:/o:redhat:enterprise_linux:6",
                },
            ],
            "references": ["https://www.debian.org/security/2019/dsa-4413"],
            "name": "CVE-2019-9755",
        }

    # flake8: noqa: E501
    @pytest.fixture
    def mock_eus_cve(self):
        return {
            "threat_severity" : "Moderate",
            "public_date" : "2024-01-18T00:00:00Z",
            "bugzilla" : {
                "description" : "kernel: ext4: kernel bug in ext4_write_inline_data_end()",
                "id" : "2261976",
                "url" : "https://bugzilla.redhat.com/show_bug.cgi?id=2261976"
            },
            "cvss3" : {
                "cvss3_base_score" : "6.7",
                "cvss3_scoring_vector" : "CVSS:3.1/AV:L/AC:H/PR:L/UI:R/S:U/C:H/I:H/A:H",
                "status" : "verified"
            },
            "cwe" : "CWE-190",
            "details" : [ "Integer Overflow or Wraparound vulnerability in openEuler kernel on Linux (filesystem modules) allows Forced Integer Overflow.This issue affects openEuler kernel: from 4.19.90 before 4.19.90-2401.3, from 5.10.0-60.18.0 before 5.10.0-183.0.0.", "A flaw was found in the openEuler kernel in Linux filesystem modules that allows an integer overflow via mounting a corrupted filesystem. This issue affects the openEuler kernel in versions from 4.19.90 through 4.19.90-2401.3 and 5.10.0-60.18.0 through 5.10.0-183.0.0." ],
            "statement" : "Red Hat has protection mechanisms in place against buffer overflows, such as FORTIFY_SOURCE, Position Independent Executables or Stack Smashing Protection.",
            "affected_release" : [ {
                "product_name" : "Red Hat Enterprise Linux 8",
                "release_date" : "2024-04-02T00:00:00Z",
                "advisory" : "RHSA-2024:1614",
                "cpe" : "cpe:/a:redhat:enterprise_linux:8::nfv",
                "package" : "kernel-rt-0:4.18.0-513.24.1.rt7.326.el8_9"
            }, {
                "product_name" : "Red Hat Enterprise Linux 8",
                "release_date" : "2024-04-02T00:00:00Z",
                "advisory" : "RHSA-2024:1607",
                "cpe" : "cpe:/o:redhat:enterprise_linux:8",
                "package" : "kernel-0:4.18.0-513.24.1.el8_9"
            }, {
                "product_name" : "Red Hat Enterprise Linux 8.6 Extended Update Support",
                "release_date" : "2024-04-03T00:00:00Z",
                "advisory" : "RHSA-2024:1653",
                "cpe" : "cpe:/o:redhat:rhel_eus:8.6",
                "package" : "kernel-0:4.18.0-372.98.1.el8_6"
            }, {
                "product_name" : "Red Hat Enterprise Linux 8.8 Extended Update Support",
                "release_date" : "2024-04-30T00:00:00Z",
                "advisory" : "RHSA-2024:2621",
                "cpe" : "cpe:/o:redhat:rhel_eus:8.8",
                "package" : "kernel-0:4.18.0-477.55.1.el8_8"
            }, {
                "product_name" : "Red Hat Enterprise Linux 9",
                "release_date" : "2023-05-09T00:00:00Z",
                "advisory" : "RHSA-2023:2458",
                "cpe" : "cpe:/a:redhat:enterprise_linux:9",
                "package" : "kernel-0:5.14.0-284.11.1.el9_2"
            }, {
                "product_name" : "Red Hat Enterprise Linux 9",
                "release_date" : "2023-05-09T00:00:00Z",
                "advisory" : "RHSA-2023:2148",
                "cpe" : "cpe:/a:redhat:enterprise_linux:9::nfv",
                "package" : "kernel-rt-0:5.14.0-284.11.1.rt14.296.el9_2"
            }, {
                "product_name" : "Red Hat Enterprise Linux 9",
                "release_date" : "2023-05-09T00:00:00Z",
                "advisory" : "RHSA-2023:2458",
                "cpe" : "cpe:/o:redhat:enterprise_linux:9",
                "package" : "kernel-0:5.14.0-284.11.1.el9_2"
            }, {
                "product_name" : "Red Hat Enterprise Linux 9.0 Extended Update Support",
                "release_date" : "2024-04-16T00:00:00Z",
                "advisory" : "RHSA-2024:1836",
                "cpe" : "cpe:/a:redhat:rhel_eus:9.0",
                "package" : "kernel-0:5.14.0-70.97.1.el9_0"
            }, {
                "product_name" : "Red Hat Enterprise Linux 9.0 Extended Update Support",
                "release_date" : "2024-04-16T00:00:00Z",
                "advisory" : "RHSA-2024:1840",
                "cpe" : "cpe:/a:redhat:rhel_eus:9.0::nfv",
                "package" : "kernel-rt-0:5.14.0-70.97.1.rt21.169.el9_0"
            }, {
                "product_name" : "Red Hat Virtualization 4 for Red Hat Enterprise Linux 8",
                "release_date" : "2024-04-03T00:00:00Z",
                "advisory" : "RHSA-2024:1653",
                "cpe" : "cpe:/o:redhat:rhev_hypervisor:4.4::el8",
                "package" : "kernel-0:4.18.0-372.98.1.el8_6"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/cluster-logging-operator-bundle:v5.7.13-16"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/cluster-logging-rhel8-operator:v5.7.13-7"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/elasticsearch6-rhel8:v6.8.1-408"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/elasticsearch-operator-bundle:v5.7.13-19"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/elasticsearch-proxy-rhel8:v1.0.0-480"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/elasticsearch-rhel8-operator:v5.7.13-9"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/eventrouter-rhel8:v0.4.0-248"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/fluentd-rhel8:v1.14.6-215"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/kibana6-rhel8:v6.8.1-431"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/log-file-metric-exporter-rhel8:v1.1.0-228"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/logging-curator5-rhel8:v5.8.1-471"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/logging-loki-rhel8:v2.9.6-15"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/logging-view-plugin-rhel8:v5.7.13-3"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/loki-operator-bundle:v5.7.13-27"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/loki-rhel8-operator:v5.7.13-12"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/lokistack-gateway-rhel8:v0.1.0-527"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/opa-openshift-rhel8:v0.1.0-225"
            }, {
                "product_name" : "RHOL-5.7-RHEL-8",
                "release_date" : "2024-05-01T00:00:00Z",
                "advisory" : "RHSA-2024:2093",
                "cpe" : "cpe:/a:redhat:logging:5.7::el8",
                "package" : "openshift-logging/vector-rhel8:v0.28.1-57"
            } ],
            "package_state" : [ {
                "product_name" : "Red Hat Enterprise Linux 6",
                "fix_state" : "Out of support scope",
                "package_name" : "kernel",
                "cpe" : "cpe:/o:redhat:enterprise_linux:6"
            }, {
                "product_name" : "Red Hat Enterprise Linux 7",
                "fix_state" : "Out of support scope",
                "package_name" : "kernel",
                "cpe" : "cpe:/o:redhat:enterprise_linux:7"
            }, {
                "product_name" : "Red Hat Enterprise Linux 7",
                "fix_state" : "Out of support scope",
                "package_name" : "kernel-rt",
                "cpe" : "cpe:/o:redhat:enterprise_linux:7"
            } ],
            "references" : [ "https://www.cve.org/CVERecord?id=CVE-2021-33631\nhttps://nvd.nist.gov/vuln/detail/CVE-2021-33631\nhttps://seclists.org/oss-sec/2024/q1/65" ],
            "name" : "CVE-2021-33631",
            "mitigation" : {
                "value" : "Mitigation for this issue is either not available or the currently available options do not meet the Red Hat Product Security criteria comprising ease of use and deployment, applicability to widespread installation base or stability.",
                "lang" : "en:us"
            },
            "csaw" : False
        }

    # noqa: E501
    @pytest.fixture
    def mock_rhsa_dict(self):
        return {
            ("RHSA-2019:2308", "7"): (
                "632",
                {
                    "Vulnerability": {
                        "Severity": "Low",
                        "NamespaceName": "7",
                        "FixedIn": [
                            {
                                "Name": "libguestfs-winsupport",
                                "Version": "0:7.2-3.el7",
                                "VersionFormat": "rpm",
                                "NamespaceName": "7",
                            }
                        ],
                        "Link": "https://access.redhat.com/errata/RHSA-2019:2308",
                        "Description": "",
                        "Metadata": {
                            "Issued": "2019-08-06",
                            "Updated": "2019-08-06",
                            "RefId": "RHSA-2019:2308",
                            "CVE": [
                                {
                                    "Name": "CVE-2019-9755",
                                    "Link": "https://access.redhat.com/security/cve/CVE-2019-9755",
                                }
                            ],
                        },
                        "Name": "RHSA-2019:2308",
                        "CVSS": [],
                    }
                },
            )
        }

    # flake8: noqa: E501
    @pytest.fixture
    def mock_rhsa_dict_2(self):
        return {
            ("RHSA-2019:2308", "7"): (
                "632",
                {
                    "Vulnerability": {
                        "Severity": "Low",
                        "NamespaceName": "7",
                        "FixedIn": [
                            {
                                "Name": "libguestfs-winsupport",
                                "Version": "0:7.2-3.el7",
                                "VersionFormat": "rpm",
                                "NamespaceName": "7",
                            }
                        ],
                        "Link": "https://access.redhat.com/errata/RHSA-2019:2308",
                        "Name": "RHSA-2019:2308",
                    }
                },
            ),
            ("RHSA-2019:3345", "8"): (
                "765",
                {
                    "Vulnerability": {
                        "Severity": "Low",
                        "NamespaceName": "8",
                        "FixedIn": [
                            {
                                "Name": "libguestfs-winsupport",
                                "Version": "0:7.2-3.el8",
                                "VersionFormat": "rpm",
                                "NamespaceName": "8",
                            }
                        ],
                        "Link": "https://access.redhat.com/errata/RHSA-2019:3345",
                        "Name": "RHSA-2019:3345",
                    }
                },
            ),
        }

    # flake8: noqa: E501
    @pytest.fixture
    def mock_cve_partial_fix(self):
        return {
            "threat_severity": "Important",
            "public_date": "2017-11-24T00:00:00Z",
            "bugzilla": {
                "description": "CVE-2017-16939 Kernel: ipsec: xfrm: use-after-free leading to potential privilege escalation",
                "id": "1517220",
                "url": "https://bugzilla.redhat.com/show_bug.cgi?id=1517220",
            },
            "cvss": {
                "cvss_base_score": "6.2",
                "cvss_scoring_vector": "AV:L/AC:H/Au:N/C:C/I:C/A:C",
                "status": "verified",
            },
            "cvss3": {
                "cvss3_base_score": "8.1",
                "cvss3_scoring_vector": "CVSS:3.0/AV:L/AC:H/PR:N/UI:N/S:C/C:H/I:H/A:H",
                "status": "verified",
            },
            "cwe": "CWE-416",
            "details": [
                "The XFRM dump policy implementation in net/xfrm/xfrm_user.c in the Linux kernel before 4.13.11 allows local users to gain privileges or cause a denial of service (use-after-free) via a crafted SO_RCVBUF setsockopt system call in conjunction with XFRM_MSG_GETPOLICY Netlink messages.",
                "The Linux kernel is vulerable to a use-after-free flaw when Transformation User configuration interface(CONFIG_XFRM_USER) compile-time configuration were enabled. This vulnerability occurs while closing a xfrm netlink socket in xfrm_dump_policy_done. A user/process could  abuse this flaw to potentially escalate their privileges on a system.",
            ],
            "statement": "This issue does not affect the versions of Linux kernel as shipped with Red Hat Enterprise Linux 5 and Red Hat Enterprise Linux 6.\nThis issue affects the version of the kernel package as shipped with Red Hat Enterprise Linux 7 and Red Hat Enterprise MRG 2. Future kernel updates for Red Hat Enterprise Linux 7 and Red Hat Enterprise MRG 2 may address this issue.",
            "affected_release": [
                {
                    "product_name": "Red Hat Enterprise Linux 7",
                    "release_date": "2018-05-08T00:00:00Z",
                    "advisory": "RHSA-2018:1355",
                    "cpe": "cpe:/a:redhat:rhel_extras_rt:7",
                    "package": "kernel-rt-0:3.10.0-862.2.3.rt56.806.el7",
                },
                {
                    "product_name": "Red Hat Enterprise Linux 7",
                    "release_date": "2018-05-08T00:00:00Z",
                    "advisory": "RHSA-2018:1318",
                    "cpe": "cpe:/o:redhat:enterprise_linux:7",
                    "package": "kernel-0:3.10.0-862.2.3.el7",
                },
            ],
            "package_state": [
                {
                    "product_name": "Red Hat Enterprise Linux 7",
                    "fix_state": "Affected",
                    "package_name": "kernel",
                    "cpe": "cpe:/o:redhat:enterprise_linux:7",
                }
            ],
            "name": "CVE-2017-16939",
        }

    def test_parse_affected_releases_eus(self, mock_eus_cve, tmpdir):
        driver = Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))
        driver.rhsa_provider = OVALRHSAProvider.from_rhsa_dict({})

        results = driver._parse_affected_release(mock_eus_cve.get("name"), mock_eus_cve)

        assert results and isinstance(results, list) and len(results) == 8

        # see https://access.redhat.com/security/cve/cve-2021-33631
        expected = [
            FixedIn(package='kernel-rt', platform='8', version='0:4.18.0-513.24.1.rt7.326.el8_9', module=None, advisory=Advisory(wont_fix=False, rhsa_id='RHSA-2024:1614', link='https://access.redhat.com/errata/RHSA-2024:1614', severity=None)),
            FixedIn(package='kernel', platform='8', version='0:4.18.0-513.24.1.el8_9', module=None, advisory=Advisory(wont_fix=False, rhsa_id='RHSA-2024:1607', link='https://access.redhat.com/errata/RHSA-2024:1607', severity=None)),
            FixedIn(package='kernel', platform='8.6+eus', version='0:4.18.0-372.98.1.el8_6', module=None, advisory=Advisory(wont_fix=False, rhsa_id='RHSA-2024:1653', link='https://access.redhat.com/errata/RHSA-2024:1653', severity=None)),
            FixedIn(package='kernel', platform='8.8+eus', version='0:4.18.0-477.55.1.el8_8', module=None, advisory=Advisory(wont_fix=False, rhsa_id='RHSA-2024:2621', link='https://access.redhat.com/errata/RHSA-2024:2621', severity=None)),
            FixedIn(package='kernel', platform='9', version='0:5.14.0-284.11.1.el9_2', module=None, advisory=Advisory(wont_fix=False, rhsa_id='RHSA-2023:2458', link='https://access.redhat.com/errata/RHSA-2023:2458', severity=None)),
            FixedIn(package='kernel-rt', platform='9', version='0:5.14.0-284.11.1.rt14.296.el9_2', module=None, advisory=Advisory(wont_fix=False, rhsa_id='RHSA-2023:2148', link='https://access.redhat.com/errata/RHSA-2023:2148', severity=None)),
            FixedIn(package='kernel', platform='9.0+eus', version='0:5.14.0-70.97.1.el9_0', module=None, advisory=Advisory(wont_fix=False, rhsa_id='RHSA-2024:1836', link='https://access.redhat.com/errata/RHSA-2024:1836', severity=None)),
            FixedIn(package='kernel-rt', platform='9.0+eus', version='0:5.14.0-70.97.1.rt21.169.el9_0', module=None, advisory=Advisory(wont_fix=False, rhsa_id='RHSA-2024:1840', link='https://access.redhat.com/errata/RHSA-2024:1840', severity=None)),
        ]

        assert expected == results

    def test_parse_affected_releases_0(self, mock_cve, tmpdir):
        driver = Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))
        driver.rhsa_provider = OVALRHSAProvider.from_rhsa_dict({})

        results = driver._parse_affected_release(mock_cve.get("name"), mock_cve)

        assert results and isinstance(results, list) and len(results) == 1
        fixed_in = results[0]
        assert fixed_in.platform == "7"
        assert fixed_in.package == "libguestfs-winsupport"
        assert fixed_in.version == "7.2-3.el7"
        assert fixed_in.advisory.rhsa_id == "RHSA-2019:2308"

    @pytest.mark.parametrize(
        "affected_releases,fixed_ins",
        [
            (
                {
                    "affected_release": [
                        {
                            "product_name": "Red Hat Enterprise Linux 7",
                            "advisory": "RHSA-2019:2308",
                            "package": "libguestfs-winsupport-7.2-3.el7",
                        },
                        {
                            "product_name": "Red Hat Enterprise Linux 8",
                            "advisory": "RHSA-2019:3345",
                            # "package": "virt:rhel-8010020190916153839.cdc1202b" Deliberately skipping version
                        },
                    ],
                    "name": "CVE-2019-9755",
                },
                [
                    FixedIn(
                        module=None,
                        package="libguestfs-winsupport",
                        platform="7",
                        version="0:7.2-3.el7",
                        advisory=Advisory(
                            wont_fix=False,
                            rhsa_id="RHSA-2019:2308",
                            severity=None,
                            link="https://access.redhat.com/errata/RHSA-2019:2308",
                        ),
                    ),
                    FixedIn(
                        module=None,
                        package="libguestfs-winsupport",
                        platform="8",
                        version="0:7.2-3.el8",
                        advisory=Advisory(
                            wont_fix=False,
                            rhsa_id="RHSA-2019:3345",
                            severity=None,
                            link="https://access.redhat.com/errata/RHSA-2019:3345",
                        ),
                    ),
                ],
            ),
            (
                {
                    "affected_release": [
                        {
                            "product_name": "Red Hat Enterprise Linux 7",
                            "advisory": "RHSA-2019:2308",
                            "package": "libguestfs-winsupport-7.2-3.el7",
                        },
                        {
                            "product_name": "Red Hat Enterprise Linux 8",
                            "advisory": "RHSA-2019:3345",
                            "package": "virt:rhel-8010020190916153839.cdc1202b",
                            # package name lookup using RHSA and other packages shouldn't yield any results
                        },
                    ],
                    "name": "CVE-2019-9755",
                },
                [
                    FixedIn(
                        module=None,
                        package="libguestfs-winsupport",
                        platform="7",
                        version="0:7.2-3.el7",
                        advisory=Advisory(
                            wont_fix=False,
                            rhsa_id="RHSA-2019:2308",
                            severity=None,
                            link="https://access.redhat.com/errata/RHSA-2019:2308",
                        ),
                    )
                ],
            ),
            (
                {
                    "affected_release": [
                        {
                            "product_name": "Red Hat Enterprise Linux 7",
                            "advisory": "RHSA-2019:1234",
                            "package": "libguestfs-winsupport-7.2-3.el7",
                        },
                        {
                            "product_name": "Red Hat Enterprise Linux 8",
                            "advisory": "RHSA-2019:5678",
                            "package": "virt:rhel-8010020190916153839.cdc1202b",
                            # RHSA lookup shouldn't yield results
                        },
                    ],
                    "name": "CVE-2019-9755",
                },
                [
                    FixedIn(
                        module=None,
                        package="libguestfs-winsupport",
                        platform="7",
                        version="7.2-3.el7",
                        advisory=Advisory(
                            wont_fix=False,
                            rhsa_id="RHSA-2019:1234",
                            severity=None,
                            link="https://access.redhat.com/errata/RHSA-2019:1234",
                        ),
                    )
                ],
            ),
            (
                {
                    "affected_release": [
                        {
                            "product_name": "Red Hat Enterprise Linux 7",
                            "package": "libguestfs-winsupport-7.2-3.el7",
                        }
                    ],
                    "name": "CVE-2019-9755",
                },
                [
                    FixedIn(
                        module=None,
                        package="libguestfs-winsupport",
                        platform="7",
                        version="7.2-3.el7",
                        advisory=Advisory(wont_fix=False, rhsa_id=None, link=None, severity=None),
                    )
                ],
            ),
            (
                # same package and platform, different versions - streams
                {
                    "affected_release": [
                        {
                            "product_name": "Red Hat Enterprise Linux 7",
                            "package": "libguestfs-winsupport-7.2-3.el7",
                        },
                        {
                            "product_name": "Red Hat Enterprise Linux 7",
                            "package": "libguestfs-winsupport-7.2-3.el7.1",
                        },
                    ],
                    "name": "CVE-2019-9755",
                },
                [
                    FixedIn(
                        module=None,
                        package="libguestfs-winsupport",
                        platform="7",
                        version="7.2-3.el7.1",
                        advisory=Advisory(wont_fix=False, rhsa_id=None, link=None, severity=None),
                    )
                ],
            ),
            (
                # same package and platform, different versions - major, minor
                {
                    "affected_release": [
                        {
                            "product_name": "Red Hat Enterprise Linux 8",
                            "package": "nodejs-1:11.19.1-2.module+el8.1.0+6118+5aaa808b",
                        },
                        {
                            "product_name": "Red Hat Enterprise Linux 8",
                            "package": "nodejs-1:10.19.0-2.module+el8.1.0+6118+5aaa808b",
                        },
                        {
                            "product_name": "Red Hat Enterprise Linux 8",
                            "package": "nodejs-1:12.16.1-2.module+el8.1.0+6117+b25a342c",
                        },
                    ],
                    "name": "CVE-2019-9755",
                },
                [
                    FixedIn(
                        module=None,
                        package="nodejs",
                        platform="8",
                        version="1:12.16.1-2.module+el8.1.0+6117+b25a342c",
                        advisory=Advisory(wont_fix=False, rhsa_id=None, link=None, severity=None),
                    )
                ],
            ),
            (
                # same package and platform, different versions - release, stream
                {
                    "affected_release": [
                        {
                            "product_name": "Red Hat Enterprise Linux 6",
                            "package": "qemu-kvm-2:0.12.1.2-2.160.el6_1.9",
                        },
                        {
                            "product_name": "Red Hat Enterprise Linux 6",
                            "package": "qemu-kvm-2:0.12.1.2-2.209.el6_2.1",
                        },
                    ],
                    "name": "CVE-2019-9755",
                },
                [
                    FixedIn(
                        module=None,
                        package="qemu-kvm",
                        platform="6",
                        version="2:0.12.1.2-2.209.el6_2.1",
                        advisory=Advisory(wont_fix=False, rhsa_id=None, link=None, severity=None),
                    )
                ],
            ),
        ],
        ids=["case1" , "case2", "case3", "case4", "case5", "case6", "case7"],
    )
    def test_parse_affected_releases(self, tmpdir, affected_releases, fixed_ins, mock_rhsa_dict_2):
        driver = Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))
        driver.rhsa_provider = OVALRHSAProvider.from_rhsa_dict(mock_rhsa_dict_2)

        results = driver._parse_affected_release(affected_releases.get("name"), affected_releases)
        assert isinstance(results, list)
        results.sort(key=lambda x: x.platform)
        fixed_ins.sort(key=lambda x: x.platform)
        assert results == fixed_ins

    def test_parse_package_state(self, tmpdir, mock_cve):
        driver = Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))
        results = driver._parse_package_state(mock_cve.get("name"), mock_cve)

        assert results and isinstance(results, list) and len(results) == 1
        fixed_in = results[0]
        assert fixed_in.platform == "6"
        assert fixed_in.package == "libguestfs-winsupport"
        assert fixed_in.version == "None"
        assert fixed_in.advisory.wont_fix is True

    def test_parse_cve(self, tmpdir, mock_cve):
        driver = Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))
        driver.rhsa_provider = OVALRHSAProvider.from_rhsa_dict({})

        results = driver._parse_cve(mock_cve.get("name"), mock_cve)
        assert results and isinstance(results, list) and len(results) == 2
        assert sorted([item.namespace for item in results]) == ["rhel:6", "rhel:7"]
        payloads = [item.payload.get("Vulnerability", {}) for item in results]
        assert all(payload.get("Name") == mock_cve.get("name") for payload in payloads)
        assert all(payload.get("Severity") == "Low" for payload in payloads)

    def test_parse_cve_partial_fix(self, tmpdir, mock_cve_partial_fix):
        driver = Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))
        driver.rhsa_provider = OVALRHSAProvider.from_rhsa_dict({})

        results = driver._parse_cve(mock_cve_partial_fix.get("name"), mock_cve_partial_fix)
        assert results and isinstance(results, list) and len(results) == 1
        fixed_ins = results[0].payload.get("Vulnerability").get("FixedIn")
        assert fixed_ins and isinstance(fixed_ins, list) and len(fixed_ins) == 2
        assert sorted([item.get("Name") for item in fixed_ins]) == [
            "kernel",
            "kernel-rt",
        ]
        kernel_record = next(item for item in fixed_ins if item.get("Name") == "kernel")
        assert kernel_record and kernel_record.get("Version") == "None"

    @pytest.mark.parametrize(
        "test_id,test_p,test_pkg,expected",
        [
            ("RHSA-2019:2308", "7", "libguestfs-winsupport", ("0:7.2-3.el7", None)),
            ("foo", "0", "bar", (None, None)),
        ],
    )
    def test_fetch_rhsa_fix_version(self, tmpdir, mock_rhsa_dict, test_id, test_p, test_pkg, expected):
        driver = Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))
        driver.rhsa_provider = OVALRHSAProvider.from_rhsa_dict(mock_rhsa_dict)

        ar_obj = AffectedRelease(rhsa_id=test_id, platform=test_p, name=test_pkg)

        assert driver._fetch_rhsa_fix_version("cve-123-123", ar_obj) == expected

    @pytest.mark.parametrize(
        "package,name,version",
        [
            ("haproxy-0:1.8.15-6.el8_1.1", "haproxy", "0:1.8.15-6.el8_1.1"),
            (
                "java-1.8.0-openjdk-1:1.8.0.252.b09-2.el8_1",
                "java-1.8.0-openjdk",
                "1:1.8.0.252.b09-2.el8_1",
            ),
            ("container-tools:rhel8-801002019092709091", "container-tools", None),
            ("0:libsndfile-1.0.28-10.el8", "libsndfile", "0:1.0.28-10.el8"),
            ("kernel-rt", "kernel-rt", None),
            ("libjpeg-6b-38", "libjpeg", "6b-38"),
        ],
    )
    def test_get_name_version(self, package, name, version):
        assert Parser._get_name_version(package) == (name, version)


def test_provider_schema(helpers, disable_get_requests, monkeypatch):
    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/oval/input",
    )

    c = Config()
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(root=workspace.root, config=c)

    def mock_sync_cves(*args, **kwargs):
        return os.path.join(p.parser.cve_dir_path, p.parser.__full_dir_name__)

    def mock_init_rhsa_data(*args, **kwargs):
        p.parser.rhsa_provider = OVALRHSAProvider.from_rhsa_dict({})

    monkeypatch.setattr(p.parser, "_sync_cves", mock_sync_cves)
    monkeypatch.setattr(p.parser, "_init_rhsa_data", mock_init_rhsa_data)

    p.update(None)

    assert workspace.num_result_entries() == 70
    # < test results directory >
    # ├── rhel:5
    # │   ├── cve-2017-3509.json
    # │   ├── cve-2017-3511.json
    # │   ├── cve-2017-3526.json
    # │   ├── cve-2017-3533.json
    # │   ├── cve-2017-3539.json
    # │   └── cve-2017-3544.json
    # ├── rhel:6
    # │   ├── cve-2017-3509.json
    # │   ├── cve-2017-3511.json
    # │   ├── cve-2017-3526.json
    # │   ├── cve-2017-3533.json
    # │   ├── cve-2017-3539.json
    # │   ├── cve-2017-3544.json
    # │   ├── cve-2020-16587.json
    # │   ├── cve-2020-16588.json
    # │   ├── cve-2021-20298.json
    # │   ├── cve-2021-20299.json
    # │   ├── cve-2022-1921.json
    # │   ├── cve-2022-1922.json
    # │   ├── cve-2022-1923.json
    # │   ├── cve-2022-1924.json
    # │   ├── cve-2022-1925.json
    # │   ├── cve-2023-4863.json
    # │   ├── cve-2023-5129.json
    # │   └── cve-2023-5217.json
    # ├── rhel:7
    # │   ├── cve-2017-3509.json
    # │   ├── cve-2017-3511.json
    # │   ├── cve-2017-3526.json
    # │   ├── cve-2017-3533.json
    # │   ├── cve-2017-3539.json
    # │   ├── cve-2017-3544.json
    # │   ├── cve-2020-16587.json
    # │   ├── cve-2020-16588.json
    # │   ├── cve-2021-20298.json
    # │   ├── cve-2021-20299.json
    # │   ├── cve-2022-1921.json
    # │   ├── cve-2022-1922.json
    # │   ├── cve-2022-1923.json
    # │   ├── cve-2022-1924.json
    # │   ├── cve-2022-1925.json
    # │   ├── cve-2023-4863.json
    # │   ├── cve-2023-5129.json
    # │   └── cve-2023-5217.json
    # ├── rhel:8
    # │   ├── cve-2019-25059.json
    # │   ├── cve-2020-16587.json
    # │   ├── cve-2021-20298.json
    # │   ├── cve-2021-20299.json
    # │   ├── cve-2022-1921.json
    # │   ├── cve-2022-1922.json
    # │   ├── cve-2022-1923.json
    # │   ├── cve-2022-1924.json
    # │   ├── cve-2022-1925.json
    # │   ├── cve-2023-4863.json
    # │   ├── cve-2023-5129.json
    # │   └── cve-2023-5217.json
    # ├── rhel:8.6+eus
    # │   ├── cve-2023-4863.json
    # │   ├── cve-2023-5129.json
    # │   └── cve-2023-5217.json
    # ├── rhel:9
    # │   ├── cve-2019-25059.json
    # │   ├── cve-2022-1921.json
    # │   ├── cve-2022-1922.json
    # │   ├── cve-2022-1923.json
    # │   ├── cve-2022-1924.json
    # │   ├── cve-2022-1925.json
    # │   ├── cve-2022-2309.json
    # │   ├── cve-2023-4863.json
    # │   ├── cve-2023-5129.json
    # │   └── cve-2023-5217.json
    # └── rhel:9.0+eus
    #     ├── cve-2023-4863.json
    #     ├── cve-2023-5129.json
    #     └── cve-2023-5217.json

    assert workspace.result_schemas_valid(require_entries=True)


def test_provider_via_snapshot(helpers, disable_get_requests, monkeypatch):
    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/oval/input",
    )

    c = Config()
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    c.rhsa_source = "OVAL"
    p = Provider(root=workspace.root, config=c)

    def mock_sync_cves(*args, **kwargs):
        return os.path.join(p.parser.cve_dir_path, p.parser.__full_dir_name__)

    def mock_init_rhsa_data(*args, **kwargs):
        p.parser.rhsa_provider = OVALRHSAProvider.from_rhsa_dict({})

        return {}

    monkeypatch.setattr(p.parser, "_sync_cves", mock_sync_cves)
    monkeypatch.setattr(p.parser, "_init_rhsa_data", mock_init_rhsa_data)

    p.update(None)

    workspace.assert_result_snapshots()


@patch("vunnel.providers.rhel.Parser._sync_cves")
def test_rhel_provider_supports_skip_download(mock_sync_cves, helpers):
    assert Provider.supports_skip_download()

    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/csaf/input",
    )

    mock_sync_cves.side_effect = RuntimeError("should not be called")

    c = Config()
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    c.runtime.skip_download = True
    c.rhsa_source = "CSAF"
    p = Provider(root=workspace.root, config=c)

    with pytest.raises(RuntimeError) as e:
        p.update(None)
        assert e.match("skip download used on empty workspace")
    assert mock_sync_cves.call_count == 0

@patch("vunnel.providers.rhel.parser.http.get")
def test_rhel_provider_supports_ignore_hydra_errors(mock_http_get, helpers):

    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/csaf/input",
    )

    mock_http_get.side_effect = RuntimeError("simulate HTTP error")

    c = Config()
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    c.ignore_hydra_errors = True
    c.rhsa_source = "CSAF"
    p = Provider(root=workspace.root, config=c)

    # don't do unnecessary work in the sync
    p.parser.enumerate_minimal_cve_pages = lambda: []

    # succeed with results from the cache
    p.parser._sync_cves()

    c.ignore_hydra_errors = False
    p = Provider(root=workspace.root, config=c)

    # API failures result in sync failure
    with pytest.raises(RuntimeError) as e:
        p.parser._sync_cves()



from __future__ import annotations

import copy
import logging
import os
import re
import shutil
from html.parser import HTMLParser
from typing import TYPE_CHECKING

import yaml

from vunnel.utils import http_wrapper as http
from vunnel.utils.vulnerability import build_reference_links, vulnerability_element

if TYPE_CHECKING:
    import requests

    from vunnel import workspace

namespace = "alpine"
feedtype = "vulnerabilities"
purge_unreported = True

# read and connect time out for requests.get
requests_timeout = 125

# driver workspace
driver_workspace = None
ignore_links = {"last-update", "license.txt"}


class SecdbLandingParser(HTMLParser):
    _valid_link_ = re.compile(r"[^\.+]")  # match anything that doesn't start with one or more '.'

    def __init__(self):
        self.links = []
        super().__init__()

    def handle_starttag(self, tag, attrs):
        for attr, value in attrs:
            if attr == "href" and SecdbLandingParser._valid_link_.match(value):  # noqa: SIM102
                if value not in ignore_links:
                    self.links.append(value)
                    break


class Parser:
    _url_ = "https://secdb.alpinelinux.org"
    _secdb_dir_ = "secdb"
    _db_types = ["main", "community"]  # noqa: RUF012
    _release_regex_ = re.compile(r"v([0-9]+.[0-9]+)")
    _link_finder_regex_ = re.compile(r'href\s*=\s*"([^\.+].*)"')
    _security_reference_url_ = "https://security.alpinelinux.org/vuln"

    def __init__(
        self,
        workspace: workspace.Workspace,
        logger: logging.Logger | None = None,
        download_timeout: int = 125,
        url: str | None = None,
        security_reference_url: str | None = None,
    ):
        self.download_timeout = download_timeout
        self.source_dir_path = os.path.join(
            workspace.input_path,
            self._secdb_dir_,
            "alpine-secdb-master",
        )  # no longer used except for cleanup, leaving it here for backwards compatibility
        self.secdb_dir_path = os.path.join(workspace.input_path, self._secdb_dir_)
        self.metadata_url = url.strip("/") if url else Parser._url_
        self.security_reference_url = security_reference_url.strip("/") if security_reference_url else Parser._security_reference_url_
        if logger is None:
            logger = logging.getLogger(self.__class__.__name__)
        self.logger = logger
        self._urls = set()

    @property
    def urls(self) -> list[str]:
        return list(self._urls)

    def build_reference_links(self, vulnerability_id: str) -> list[str]:
        urls = []
        if self.security_reference_url and vulnerability_id.startswith("CVE-"):
            urls.append(f"{self.security_reference_url}/{vulnerability_id}")

        urls.extend(build_reference_links(vulnerability_id))
        return urls

    def _download(self):  # noqa: C901, PLR0912
        """
        Downloads alpine sec db files
        :return:
        """
        # remove old source directory since its no longer used
        if os.path.exists(self.source_dir_path):
            shutil.rmtree(self.source_dir_path)
        if os.path.exists(os.path.join(self.secdb_dir_path, "alpine-secdb-master.tar.gz")):
            os.remove(os.path.join(self.secdb_dir_path, "alpine-secdb-master.tar.gz"))

        links = []
        try:
            if not os.path.exists(self.secdb_dir_path):
                os.makedirs(self.secdb_dir_path, exist_ok=True)

            self.logger.info(f"downloading alpine secdb metadata from: {self.metadata_url}")
            r = self._download_metadata_url()
            try:
                self.logger.debug("HTML parsing secdb landing page content for links")
                parser = SecdbLandingParser()
                parser.feed(r.text)
                links = parser.links
            except Exception:
                self.logger.warning("unable to html parse secdb landing page content for links", exc_info=True)

            if not links:
                self.logger.debug("string parsing secdb landing page content for links")
                links = re.findall(self._link_finder_regex_, r.text)
        except Exception:
            self.logger.exception("error downloading or parsing alpine secdb metadata")
            raise

        if links:
            self.logger.debug(f"found release specific secdb links: {links}")
        else:
            raise Exception("unable to find release specific secdb links")

        for link in links:
            if link not in ignore_links:
                try:
                    rel = link.strip("/")
                    rel_dir = os.path.join(self.secdb_dir_path, rel)
                    os.makedirs(rel_dir, exist_ok=True)
                    for db_type in self._db_types:
                        if db_type == "community" and rel == "v3.2":
                            # edge case: community.yaml processing was added in v3.3
                            # since this involves semver processing and we have a known list, the simplest
                            # approach is to exclude this single version for now.
                            # future enhancement could add semver processing for something like "<3.3 continue"
                            continue

                        file_name = f"{db_type}.yaml"
                        download_url = "/".join([self.metadata_url, rel, file_name])

                        self.logger.info(f"Downloading secdb {rel} {db_type}")
                        r = self._download_url(download_url)

                        file_path = os.path.join(rel_dir, file_name)
                        with open(file_path, "wb") as fp:
                            for chunk in r.iter_content():
                                fp.write(chunk)

                except KeyboardInterrupt:
                    raise
                except Exception:
                    self.logger.exception(f"ignoring error processing secdb for {link}")

    def _download_metadata_url(self) -> requests.Response:
        return http.get(self.metadata_url, self.logger, timeout=self.download_timeout)

    def _download_url(self, url) -> requests.Response:
        self._urls.add(url)
        return http.get(url, self.logger, stream=True, timeout=self.download_timeout)

    def _load(self):
        """
        Loads all db yamls for a release and yield it
        :return:
        """
        # parse and transform the yaml
        try:
            if os.path.exists(self.secdb_dir_path):
                for f in sorted(os.listdir(self.secdb_dir_path)):
                    release = None

                    if f == "edge":
                        release = f
                    else:
                        match = self._release_regex_.match(f)
                        if match:
                            release = match.group(1)

                    if release:
                        dbtype_data_dict = {}

                        for dbtype in self._db_types:
                            secdb_yaml_path = os.path.join(self.secdb_dir_path, f, f"{dbtype}.yaml")
                            if os.path.exists(secdb_yaml_path):
                                self.logger.debug(f"loading secdb data from: {secdb_yaml_path}")
                                with open(secdb_yaml_path) as FH:
                                    yaml_data = yaml.safe_load(FH)
                                    dbtype_data_dict[dbtype] = yaml_data

                        yield release, dbtype_data_dict

            else:
                raise Exception("Cannot find alpine sec db source ")
        except Exception:
            self.logger.exception("Failed to load alpine sec db data")
            raise

    def _normalize(self, release, dbtype_data_dict):  # noqa: C901, PLR0912
        """
        Normalize all the sec db entries into vulnerability payload records
        :param release:
        :param dbtype_data_dict:
        :return:
        """

        vuln_dict = {}

        for dbtype, data in dbtype_data_dict.items():
            self.logger.info(f"processing {release}:{dbtype}")

            if data["packages"]:
                for el in data["packages"]:
                    pkg_el = el["pkg"]

                    pkg = pkg_el["name"]
                    for pkg_version in pkg_el["secfixes"]:
                        vids = []
                        if pkg_el["secfixes"][pkg_version]:
                            for rawvid in pkg_el["secfixes"][pkg_version]:
                                tmp = rawvid.split()
                                for newvid in tmp:
                                    if newvid not in vids:
                                        vids.append(newvid)

                        for vid in vids:
                            if not re.match("^CVE-.*", vid):
                                # skip non-CVE records
                                continue

                            if vid not in vuln_dict:
                                # create a new record
                                vuln_dict[vid] = copy.deepcopy(vulnerability_element)
                                vuln_record = vuln_dict[vid]
                                reference_links = self.build_reference_links(vid)

                                # populate the static information about the new vuln record
                                vuln_record["Vulnerability"]["Name"] = str(vid)
                                vuln_record["Vulnerability"]["NamespaceName"] = namespace + ":" + str(release)

                                if reference_links:
                                    vuln_record["Vulnerability"]["Link"] = reference_links[0]
                                vuln_record["Vulnerability"]["Severity"] = "Unknown"
                            else:
                                vuln_record = vuln_dict[vid]

                            # SET UP fixedins
                            fixed_el = {}
                            fixed_el["VersionFormat"] = "apk"
                            fixed_el["NamespaceName"] = namespace + ":" + str(release)
                            fixed_el["Name"] = pkg
                            fixed_el["Version"] = pkg_version

                            vuln_record["Vulnerability"]["FixedIn"].append(fixed_el)

        return vuln_dict

    def get(self):
        """
        Download, load and normalize alpine sec db and return a dict of releae - list of vulnerability records
        :return:
        """
        # download the data
        self._download()

        for release, dbtype_data_dict in self._load():
            # normalize the loaded data
            yield release, self._normalize(release, dbtype_data_dict)



from __future__ import annotations

import os.path
import shutil
from unittest.mock import MagicMock, patch

import pytest
from vunnel import result, workspace
from vunnel.providers.debian import Config, Provider, parser


@pytest.fixture()
def mock_legacy_db(mocker):
    mock_record = {
        "schema": "https://raw.githubusercontent.com/anchore/vunnel/main/schema/vulnerability/os/schema-1.0.0.json",
        "identifier": "debian:10/cve-2012-0833",
        "item": {
            "Vulnerability": {
                "Severity": "Negligible",
                "NamespaceName": "debian:10",
                "FixedIn": [],
                "Link": "https://security-tracker.debian.org/tracker/CVE-2012-0833",
                "Description": "The acllas__handle_group_entry function in servers/plugins/acl/acllas.c in 389 Directory Server before 1.2.10 does not properly handled access control instructions (ACIs) that use certificate groups, which allows remote authenticated LDAP users with a certificate group to cause a denial of service (infinite loop and CPU consumption) by binding to the server.",
                "Metadata": {},
                "Name": "CVE-2012-0833",
                "CVSS": [],
            }
        },
    }

    mock_records = [result.Envelope(**mock_record)]

    mocker.patch("vunnel.result.SQLiteReader.read_all", return_value=mock_records)


class TestParser:
    _sample_dsa_data_ = "test-fixtures/input/DSA"
    _sample_json_data_ = "test-fixtures/input/debian.json"

    def test_normalize_dsa_list(self, tmpdir, helpers, disable_get_requests):
        subject = parser.Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))

        mock_data_path = helpers.local_dir(self._sample_dsa_data_)
        shutil.copy(mock_data_path, subject.dsa_file_path)

        ns_cve_dsalist = subject._normalize_dsa_list()
        assert isinstance(ns_cve_dsalist, dict)

        assert len(ns_cve_dsalist) > 0

        for _ns, cve_dsalist in ns_cve_dsalist.items():
            assert isinstance(cve_dsalist, dict)
            assert len(cve_dsalist) > 0
            assert all(isinstance(x, list) and len(x) > 0 for x in cve_dsalist.values())

            # print("Number of CVEs in {}: {}".format(ns, len(cve_dsalist)))
            # more_dsas = {x: y for x, y in cve_dsalist.items() if len(y) > 1}
            # print("Number of CVEs with more than 1 DSA: {}".format(len(more_dsas)))
            # # for cve, dsalist in sub.items():
            # #     print('{} in debian:{} namespace is mapped to {} DSAs. {}'.format(cve, ns, len(dsalist), dsalist))
            # print("")

    def test_get_dsa_map(self, tmpdir, helpers, disable_get_requests):
        subject = parser.Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))

        mock_data_path = helpers.local_dir(self._sample_dsa_data_)
        shutil.copy(mock_data_path, subject.dsa_file_path)

        dsa_map = subject._get_dsa_map()
        # dsas = {dsa["id"] for dsa_collection in dsa_map.values() for dsa in (dsa_collection.cves + dsa_collection.nocves)}
        # print("")
        # print("Total number of dsas: {}".format(len(dsas)))

        no_cves = [dsa for dsa_collection in dsa_map.values() for dsa in dsa_collection.nocves]
        weird_dsas = [dsa for dsa in no_cves if not dsa["fixed_in"]]
        # print("")
        # print("Number of DSAs with neither fixes nor CVEs: {}".format(len(weird_dsas)))
        assert len(weird_dsas) == 3

        no_cve_dsas = [dsa for dsa in no_cves if dsa["fixed_in"]]
        # print("")
        # print("Number of DSAs with fixes and no CVEs: {}".format(len(no_cve_dsas)))
        assert len(no_cve_dsas) == 1

    def test_normalize_json(self, tmpdir, helpers, disable_get_requests):
        subject = parser.Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))
        subject.logger = MagicMock()

        dsa_mock_data_path = helpers.local_dir(self._sample_dsa_data_)
        json_mock_data_path = helpers.local_dir(self._sample_json_data_)
        shutil.copy(dsa_mock_data_path, subject.dsa_file_path)
        shutil.copy(json_mock_data_path, subject.json_file_path)

        ns_cve_dsalist = subject._normalize_dsa_list()
        vuln_records = subject._normalize_json(ns_cve_dsalist=ns_cve_dsalist)

        assert isinstance(vuln_records, dict)
        assert len(vuln_records) > 0

        for _rel, vuln_dict in vuln_records.items():
            assert isinstance(vuln_dict, dict)
            assert len(vuln_dict) > 0
            assert all("Vulnerability" in x for x in vuln_dict.values())

            assert all(x.get("Vulnerability", {}).get("Name") for x in vuln_dict.values())

            assert all(x.get("Vulnerability", {}).get("Description") is not None for x in vuln_dict.values())
        assert not subject.logger.exception.called, "no exceptions should be logged"

    def test_get_legacy_records(self, tmpdir, helpers, disable_get_requests, mock_legacy_db):
        subject = parser.Parser(workspace=workspace.Workspace(tmpdir, "test", create=True))

        mock_data_path = helpers.local_dir("test-fixtures/input")
        shutil.copytree(mock_data_path, subject.workspace.input_path, dirs_exist_ok=True)

        legacy_records = subject._get_legacy_records()

        assert isinstance(legacy_records, dict)
        assert len(legacy_records) > 0

        # from the feed service data dump
        assert "7" in legacy_records.keys()
        assert len(legacy_records["7"]) > 0
        assert "CVE-2004-1653" in legacy_records["7"].keys()
        assert len(legacy_records["7"]["CVE-2004-1653"]) > 0

        # from the DB
        assert "10" in legacy_records.keys()
        assert len(legacy_records["10"]) > 0
        assert "CVE-2012-0833" in legacy_records["10"].keys()
        assert len(legacy_records["10"]["CVE-2012-0833"]) > 0

        for _rel, vuln_dict in legacy_records.items():
            assert isinstance(vuln_dict, dict)
            assert len(vuln_dict) > 0
            assert all("Vulnerability" in x for x in vuln_dict.values())

            assert all(x.get("Vulnerability", {}).get("Name") for x in vuln_dict.values())

            assert all(x.get("Vulnerability", {}).get("Description") is not None for x in vuln_dict.values())


def test_provider_schema(helpers, disable_get_requests, monkeypatch, mock_legacy_db):
    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/input",
    )

    c = Config()
    # keep all of the default values for the result store, but override the strategy
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(
        root=workspace.root,
        config=c,
    )

    def mock_download():
        return None

    monkeypatch.setattr(p.parser, "_download_json", mock_download)
    monkeypatch.setattr(p.parser, "_download_dsa", mock_download)

    p.update(None)

    # 18 entries from the legacy FS records, 1 from legacy DB record, 21 from the mock json data
    expected = 39
    assert workspace.num_result_entries() == expected
    assert workspace.result_schemas_valid(require_entries=True)


def test_provider_via_snapshot(helpers, disable_get_requests, monkeypatch, mock_legacy_db):
    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/input",
    )

    c = Config()
    # keep all of the default values for the result store, but override the strategy
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(
        root=workspace.root,
        config=c,
    )

    def mock_download():
        return None

    monkeypatch.setattr(p.parser, "_download_json", mock_download)
    monkeypatch.setattr(p.parser, "_download_dsa", mock_download)

    p.update(None)

    workspace.assert_result_snapshots()



from __future__ import annotations

import json
import os
import shutil

import pytest
from vunnel import result, workspace
from vunnel.providers.chainguard import Config, Provider
from vunnel.providers.wolfi import parser


def test_provider_schema(helpers, disable_get_requests):
    workspace = helpers.provider_workspace_helper(name=Provider.name())

    c = Config()
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(root=workspace.root, config=c)

    mock_data_path = helpers.local_dir("test-fixtures/input")
    shutil.copytree(mock_data_path, workspace.input_dir, dirs_exist_ok=True)

    p.update(None)

    assert workspace.num_result_entries() == 189
    assert workspace.result_schemas_valid(require_entries=True)


def test_provider_via_snapshot(helpers, disable_get_requests, monkeypatch):
    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/input",
    )

    c = Config()
    # keep all of the default values for the result store, but override the strategy
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(
        root=workspace.root,
        config=c,
    )

    def mock_download():
        return None

    monkeypatch.setattr(p.parser, "_download", mock_download)

    p.update(None)

    workspace.assert_result_snapshots()



from __future__ import annotations
import logging

import shutil

import pytest
from pytest_unordered import unordered
from vunnel import result, workspace, utils
from vunnel.providers.mariner import Config, Provider, parser
from vunnel.providers.mariner.parser import MarinerXmlFile
from vunnel.utils.vulnerability import Vulnerability, FixedIn, VendorAdvisory


@pytest.mark.parametrize(
    ("input_file", "expected"),
    [
        (
            "test-fixtures/mariner-truncated-2.0-oval.xml",
            [
                Vulnerability(
                    Name="CVE-2023-29404",
                    NamespaceName="mariner:2.0",
                    Description="CVE-2023-29404 affecting package golang for versions less than 1.20.7-1. A patched version of the package is available.",
                    Severity="Critical",
                    Link="https://nvd.nist.gov/vuln/detail/CVE-2023-29404",
                    CVSS=[],
                    FixedIn=[
                        FixedIn(
                            Name="golang",
                            NamespaceName="mariner:2.0",
                            VersionFormat="rpm",
                            Version="0:1.20.7-1.cm2",
                            Module=None,
                            VendorAdvisory=None,
                            VulnerableRange="> 0:1.19.0.cm2, < 0:1.20.7-1.cm2",
                        )
                    ],
                    Metadata={},
                ),
                Vulnerability(
                    Name="CVE-2023-21980",
                    NamespaceName="mariner:2.0",
                    Description="CVE-2023-21980 affecting package mysql 8.0.32-1. An upgraded version of the package is available that resolves this issue.",
                    Severity="High",
                    Link="https://nvd.nist.gov/vuln/detail/CVE-2023-21980",
                    CVSS=[],
                    FixedIn=[
                        FixedIn(
                            Name="mysql",
                            NamespaceName="mariner:2.0",
                            VersionFormat="rpm",
                            Version="0:8.0.33-1.cm2",
                            VulnerableRange="< 0:8.0.33-1.cm2",
                            Module=None,
                            VendorAdvisory=None,
                        )
                    ],
                    Metadata={},
                ),
                Vulnerability(
                    Name="CVE-2023-21977",
                    NamespaceName="mariner:2.0",
                    Description="CVE-2023-21977 affecting package mysql 8.0.32-1. An upgraded version of the package is available that resolves this issue.",
                    Severity="Medium",
                    Link="https://nvd.nist.gov/vuln/detail/CVE-2023-21977",
                    CVSS=[],
                    FixedIn=[
                        FixedIn(
                            Name="mysql",
                            NamespaceName="mariner:2.0",
                            VersionFormat="rpm",
                            Version="0:8.0.33-1.cm2",
                            Module=None,
                            VendorAdvisory=None,
                            VulnerableRange="< 0:8.0.33-1.cm2",
                        )
                    ],
                    Metadata={},
                ),
                Vulnerability(
                    Name="CVE-2022-3736",
                    NamespaceName="mariner:2.0",
                    Description="CVE-2022-3736 affecting package bind 9.16.33-1. No patch is available currently.",
                    Severity="High",
                    Link="https://nvd.nist.gov/vuln/detail/CVE-2022-3736",
                    CVSS=[],
                    FixedIn=[
                        FixedIn(
                            Name="bind",
                            NamespaceName="mariner:2.0",
                            VersionFormat="rpm",
                            Version="None",
                            Module=None,
                            VendorAdvisory=None,
                            VulnerableRange="<= 0:9.16.33-1.cm2",
                        ),
                    ],
                ),
            ],
        ),
        (
            "test-fixtures/azure-linux-truncated-3.0-oval.xml",
            [
                Vulnerability(
                    Name="CVE-2024-24258",
                    NamespaceName="mariner:3.0",
                    Description="CVE-2024-24258 affecting package freeglut for versions less than 3.4.0-1. A patched version of the package is available.",
                    Severity="High",
                    Link="https://nvd.nist.gov/vuln/detail/CVE-2024-24258",
                    CVSS=[],
                    FixedIn=[
                        FixedIn(
                            Name="freeglut",
                            NamespaceName="mariner:3.0",
                            VersionFormat="rpm",
                            Version="0:3.4.0-1.azl3",
                            VulnerableRange="< 0:3.4.0-1.azl3",
                            Module="",
                            VendorAdvisory=VendorAdvisory(NoAdvisory=False, AdvisorySummary=[]),
                        )
                    ],
                    Metadata={},
                ),
                Vulnerability(
                    Name="CVE-2024-24259",
                    NamespaceName="mariner:3.0",
                    Description="CVE-2024-24259 affecting package freeglut for versions less than 3.4.0-1. A patched version of the package is available.",
                    Severity="High",
                    Link="https://nvd.nist.gov/vuln/detail/CVE-2024-24259",
                    CVSS=[],
                    FixedIn=[
                        FixedIn(
                            Name="freeglut",
                            NamespaceName="mariner:3.0",
                            VersionFormat="rpm",
                            Version="0:3.4.0-1.azl3",
                            VulnerableRange="< 0:3.4.0-1.azl3",
                            Module="",
                            VendorAdvisory=VendorAdvisory(NoAdvisory=False, AdvisorySummary=[]),
                        )
                    ],
                    Metadata={},
                ),
                Vulnerability(
                    Name="CVE-2020-27304",
                    NamespaceName="mariner:3.0",
                    Description="CVE-2020-27304 affecting package ceph for versions less than 18.2.1-1. An upgraded version of the package is available that resolves this issue.",
                    Severity="Critical",
                    Link="https://nvd.nist.gov/vuln/detail/CVE-2020-27304",
                    CVSS=[],
                    FixedIn=[
                        FixedIn(
                            Name="ceph",
                            NamespaceName="mariner:3.0",
                            VersionFormat="rpm",
                            Version="0:18.2.1-1.azl3",
                            VulnerableRange="< 0:18.2.1-1.azl3",
                            Module="",
                            VendorAdvisory=VendorAdvisory(NoAdvisory=False, AdvisorySummary=[]),
                        )
                    ],
                    Metadata={},
                ),
            ],
        ),
    ],
)
def test_parse(tmpdir, helpers, input_file, expected):
    mock_data_path = helpers.local_dir(input_file)
    subject = MarinerXmlFile(mock_data_path, logger=logging.getLogger("test_pariner"))

    vulnerabilities = [v for v in subject.vulnerabilities()]
    assert len(vulnerabilities) == len(expected)
    assert vulnerabilities == expected


def test_provider_schema(helpers, disable_get_requests, monkeypatch):
    workspace = helpers.provider_workspace_helper(name=Provider.name())

    c = Config(allow_versions=["2.0"])
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(root=workspace.root, config=c)

    mock_data_path = helpers.local_dir("test-fixtures/mariner-truncated-2.0-oval.xml")
    shutil.copy(mock_data_path, workspace.input_dir / "mariner-truncated-2.0-oval.xml")

    def mock_download(*args, **kwargs):
        return [mock_data_path]

    monkeypatch.setattr(p.parser, "_download", mock_download)

    p.update(None)

    assert 4 == workspace.num_result_entries()
    assert workspace.result_schemas_valid(require_entries=True)


def test_provider_via_snapshot(helpers, disable_get_requests, monkeypatch):
    workspace = helpers.provider_workspace_helper(name=Provider.name())

    c = Config(allow_versions=["2.0"])
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(root=workspace.root, config=c)

    mock_data_path = helpers.local_dir("test-fixtures/mariner-truncated-2.0-oval.xml")
    shutil.copy(mock_data_path, workspace.input_dir / "mariner-truncated-2.0-oval.xml")

    def mock_download(*args, **kwargs):
        return [mock_data_path]

    monkeypatch.setattr(p.parser, "_download", mock_download)

    p.update(None)

    workspace.assert_result_snapshots()



from __future__ import annotations

import pytest
from vunnel import provider, providers, result
from vunnel.cli import config


def test_minimal_config(helpers):
    cfg_path = helpers.local_dir("test-fixtures/minimal.yaml")
    cfg = config.load(path=cfg_path)
    assert cfg == config.Application(log=config.Log(slim=False, level="TRACE"))


def test_full_config(helpers):
    cfg_path = helpers.local_dir("test-fixtures/full.yaml")
    cfg = config.load(path=cfg_path)

    runtime_cfg = provider.RuntimeConfig(
        result_store=result.StoreStrategy.SQLITE,
        existing_input=provider.InputStatePolicy.KEEP,
        existing_results=provider.ResultStatePolicy.KEEP,
        on_error=provider.OnErrorConfig(
            action=provider.OnErrorAction.SKIP,
            retry_count=9,
            retry_delay=12,
            input=provider.InputStatePolicy.DELETE,
            results=provider.ResultStatePolicy.DELETE,
        ),
    )

    assert cfg == config.Application(
        root="/tmp/all-the-dataz",
        log=config.Log(
            slim=True,
            level="trace",
            show_level=False,
            show_timestamp=True,
        ),
        providers=config.Providers(
            alpine=providers.alpine.Config(
                runtime=runtime_cfg,
                request_timeout=20,
            ),
            amazon=providers.amazon.Config(
                security_advisories={
                    # this is what we added in the config
                    42: "https://alas.aws.amazon.com/AL2/alas-42.rss",
                    # this is the defaults...
                    2: "https://alas.aws.amazon.com/AL2/alas.rss",
                    2022: "https://alas.aws.amazon.com/AL2022/alas.rss",
                    2023: "https://alas.aws.amazon.com/AL2023/alas.rss",
                },
                runtime=runtime_cfg,
                request_timeout=20,
                max_allowed_alas_http_403=33,
            ),
            chainguard=providers.chainguard.Config(
                runtime=runtime_cfg,
                request_timeout=20,
            ),
            debian=providers.debian.Config(
                releases={
                    # this is the defaults...
                    "trixie": "13",
                    "bookworm": "12",
                    "bullseye": "11",
                    "buster": "10",
                    "stretch": "9",
                    "jessie": "8",
                    "wheezy": "7",
                    "sid": "unstable",
                    # this is what we added in the config
                    "jinx": 87,
                },
                runtime=runtime_cfg,
                request_timeout=20,
            ),
            epss=providers.epss.Config(
                runtime=runtime_cfg,
                dataset="2025-03-04",
                url_template="https://localhost/epss_scores-{}.csv.gz",
                request_timeout=20,
            ),
            github=providers.github.Config(
                token="",
                api_url="https://api.gitlab.com/graphql",
                runtime=runtime_cfg,
                request_timeout=20,
            ),
            kev=providers.kev.Config(
                runtime=runtime_cfg,
                url="https://anchore.com/kev/data.json",
                request_timeout=20,
            ),
            minimos=providers.minimos.Config(
                runtime=runtime_cfg,
                request_timeout=20,
            ),
            nvd=providers.nvd.Config(
                runtime=runtime_cfg,
                request_timeout=20,
                request_retry_count=50,
                overrides_enabled=True,
                overrides_url="https://github.com/anchore/nvd-data-overrides/SOMEWHEREELSE/main.tar.gz",
            ),
            oracle=providers.oracle.Config(
                runtime=runtime_cfg,
                request_timeout=20,
            ),
            rhel=providers.rhel.Config(
                runtime=runtime_cfg,
                request_timeout=20,
                parallelism=7,
                full_sync_interval=22,
                skip_namespaces=["rhel:8"],
            ),
            sles=providers.sles.Config(
                runtime=runtime_cfg,
                request_timeout=20,
                allow_versions=[13],
            ),
            ubuntu=providers.ubuntu.Config(
                runtime=runtime_cfg,
                request_timeout=20,
                additional_versions={"zzz": "24.24"},
                enable_rev_history=True,
                parallelism=25,
                git_url="https://xyz.abc",
                git_branch="yoda",
            ),
            wolfi=providers.wolfi.Config(
                runtime=runtime_cfg,
                request_timeout=20,
            ),
        ),
    )


@pytest.mark.parametrize(
    "common_enabled,provider_enabled,want",
    [
        # default wins if provider is None
        (True, None, True),
        (False, None, False),
        # provider always overrides default
        (True, False, False),
        (False, True, True),
        # if everything agrees, that's the answer
        (True, True, True),
        (False, False, False),
    ],
)
def test_import_results_enabled(common_enabled: bool, provider_enabled: bool | None, want: bool):
    cfg = config.Application(
        providers=config.Providers(
            common=config.CommonProviderConfig(
                import_results=config.ImportResults(
                    enabled=common_enabled,
                )
            ),
            nvd=providers.nvd.Config(
                runtime=provider.RuntimeConfig(
                    import_results_enabled=provider_enabled,
                )
            ),
        )
    )
    assert cfg.providers.nvd.runtime.import_results_enabled == want


@pytest.mark.parametrize(
    "common_path,provider_path,want",
    [
        ("default_value", None, "default_value"),
        ("default_value", "specific_value", "specific_value"),
        ("default_value", "", "default_value"),
        ("default_value", "/", "/"),
    ],
)
def test_import_results_path(common_path: str, provider_path: str | None, want: str):
    cfg = config.Application(
        providers=config.Providers(
            common=config.CommonProviderConfig(
                import_results=config.ImportResults(
                    path=common_path,
                )
            ),
            nvd=providers.nvd.Config(
                runtime=provider.RuntimeConfig(
                    import_results_path=provider_path,
                )
            ),
        )
    )
    assert cfg.providers.nvd.runtime.import_results_path == want


@pytest.mark.parametrize(
    "common_host,provider_host,want",
    [
        ("default-host", None, "default-host"),
        ("default-host", "specific-host", "specific-host"),
        ("default-host", "", "default-host"),  # TODO: should this be "default-host"?
    ],
)
def test_import_results_host(common_host: str, provider_host: str | None, want: str):
    cfg = config.Application(
        providers=config.Providers(
            common=config.CommonProviderConfig(
                import_results=config.ImportResults(
                    host=common_host,
                )
            ),
            nvd=providers.nvd.Config(
                runtime=provider.RuntimeConfig(
                    import_results_host=provider_host,
                )
            ),
        )
    )
    assert cfg.providers.nvd.runtime.import_results_host == want



from __future__ import annotations

import datetime
import logging
import os
from typing import TYPE_CHECKING, Any

from vunnel import result
from vunnel.providers.nvd.api import NvdAPI
from vunnel.providers.nvd.overrides import NVDOverrides

if TYPE_CHECKING:
    from collections.abc import Generator

    from vunnel import schema as schema_def
    from vunnel.workspace import Workspace


class Manager:
    __nvd_input_db__ = "nvd-input.db"

    def __init__(  # noqa: PLR0913
        self,
        workspace: Workspace,
        schema: schema_def.Schema,
        overrides_url: str,
        logger: logging.Logger | None = None,
        download_timeout: int = 125,
        download_retry_count: int = 10,
        api_key: str | None = None,
        overrides_enabled: bool = False,
    ) -> None:
        self.workspace = workspace

        if not logger:
            logger = logging.getLogger(self.__class__.__name__)
        self.logger = logger

        self.api = NvdAPI(api_key=api_key, logger=logger, timeout=download_timeout, retries=download_retry_count)

        self.overrides = NVDOverrides(
            enabled=overrides_enabled,
            url=overrides_url,
            workspace=workspace,
            logger=logger,
            download_timeout=download_timeout,
            retries=download_retry_count,
        )

        self.urls = [self.api._cve_api_url_]
        self.schema = schema

    def get(
        self,
        last_updated: datetime.datetime | None,
        skip_if_exists: bool = False,
    ) -> Generator[tuple[str, dict[str, Any]], Any, None]:
        self.overrides.download()

        cves_processed = set()
        for record_id, record in self._download_nvd_input(last_updated, skip_if_exists):
            cves_processed.add(id_to_cve(record_id))
            yield record_id, record

        if self.overrides.enabled:
            self.urls.append(self.overrides.url)
            self.logger.debug("applying NVD data overrides...")

            override_cves = {cve.lower() for cve in self.overrides.cves()}
            override_remaining_cves = override_cves - cves_processed
            with self._sqlite_reader() as reader:
                for cve in override_remaining_cves:
                    original_record = reader.read(cve_to_id(cve))
                    if not original_record:
                        self.logger.warning(f"override for {cve} not found in original data")
                        continue

                    original_record = original_record["item"]
                    if not original_record:
                        self.logger.warning(f"missing original data for {cve}")
                        continue

                    yield cve_to_id(cve), self._apply_override(cve, original_record)

            self.logger.debug(f"applied overrides for {len(override_remaining_cves)} CVEs")
        else:
            self.logger.debug("overrides are not enabled, skipping...")

    def _download_nvd_input(
        self,
        last_updated: datetime.datetime | None,
        skip_if_exists: bool = False,
    ) -> Generator[tuple[str, dict[str, Any]], Any, None]:
        with self._nvd_input_writer() as writer:
            if skip_if_exists and self._can_update_incrementally(last_updated):
                yield from self._download_updates(last_updated, writer)  # type: ignore  # noqa: PGH003
            else:
                yield from self._download_all(writer)

    def _nvd_input_writer(self) -> result.Writer:
        return result.Writer(
            workspace=self.workspace,
            result_state_policy=result.ResultStatePolicy.KEEP,
            logger=self.logger,
            store_strategy=result.StoreStrategy.SQLITE,
            write_location=self._input_nvd_path,
        )

    def _sqlite_reader(self) -> result.SQLiteReader:
        return result.SQLiteReader(sqlite_db_path=self._input_nvd_path)

    @property
    def _input_nvd_path(self) -> str:
        return os.path.join(self.workspace.input_path, self.__nvd_input_db__)

    def _can_update_incrementally(self, last_updated: datetime.datetime | None) -> bool:
        input_db_path = os.path.join(self.workspace.input_path, self.__nvd_input_db__)
        if not os.path.exists(input_db_path):
            return False

        if not last_updated:
            return False

        now = datetime.datetime.now(tz=datetime.UTC)
        days_since_last_sync = (now - last_updated).days

        if days_since_last_sync >= NvdAPI.max_date_range_days:
            self.logger.info(
                f"last sync was {days_since_last_sync} days ago (more than {NvdAPI.max_date_range_days} days, the max range value of the NVD API), downloading all data",  # noqa: E501
            )
            return False

        return True

    def _download_all(self, writer: result.Writer) -> Generator[tuple[str, dict[str, Any]], Any, None]:
        self.logger.info("downloading all CVEs")

        # TODO: should we delete all existing state in this case first?

        for response in self.api.cve():
            yield from self._unwrap_records(response, writer)

    def _download_updates(
        self,
        last_updated: datetime.datetime,
        writer: result.Writer,
    ) -> Generator[tuple[str, dict[str, Any]], Any, None]:
        self.logger.debug(f"downloading CVEs changed since {last_updated.isoformat()}")

        # get the list of CVEs that have been updated since the last sync
        now = datetime.datetime.now(tz=datetime.UTC)
        for idx, response in enumerate(self.api.cve(last_mod_start_date=last_updated, last_mod_end_date=now)):
            if idx == 0:
                total_results = response.get("totalResults", None)
                if total_results:
                    self.logger.debug(f"discovered {total_results} updated CVEs")

            yield from self._unwrap_records(response, writer)

    def _unwrap_records(
        self,
        response: dict[str, Any],
        writer: result.Writer,
    ) -> Generator[tuple[str, dict[str, Any]], Any, None]:
        for vuln in response["vulnerabilities"]:
            cve_id = vuln["cve"]["id"]
            record_id = cve_to_id(cve_id)

            # keep input for future overrides
            writer.write(record_id.lower(), self.schema, vuln)

            # apply overrides to output
            yield record_id, self._apply_override(cve_id=cve_id, record=vuln)

    def _apply_override(self, cve_id: str, record: dict[str, Any]) -> dict[str, Any]:
        override = self.overrides.cve(cve_id)
        if override:
            self.logger.debug(f"applying override for {cve_id}")
            # ignore empty overrides
            if override is None or "cve" not in override:
                return record
            # explicitly only support CPE configurations for now and always override the
            # original record configurations. Can figure out more complicated scenarios
            # later if needed
            if "configurations" not in override["cve"]:
                return record

            record["cve"]["configurations"] = override["cve"]["configurations"]

        return record


def cve_to_id(cve: str) -> str:
    year = cve.split("-")[1]
    return os.path.join(year, cve)


def id_to_cve(cve_id: str) -> str:
    return cve_id.split("/")[1]



from __future__ import annotations

import datetime
import pytest
import os
import shutil

from vunnel import result, schema, workspace


@pytest.fixture()
def ws(tmpdir) -> workspace.Workspace:
    return workspace.Workspace(root=tmpdir, name="nvd", create=True)


@pytest.fixture()
def flat_file_existing_workspace(ws: workspace.Workspace) -> workspace.Workspace:
    store_strategy = result.StoreStrategy.FLAT_FILE
    with result.Writer(ws, result_state_policy=result.ResultStatePolicy.DELETE, store_strategy=store_strategy) as writer:
        writer.write(
            identifier="dummy-result-1",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-1"}},
        )

        writer.write(
            identifier="dummy-result-2",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-2"}},
        )

    ws.record_state(timestamp=datetime.datetime.now(), urls=[], store=store_strategy, version=1, distribution_version=1)
    return ws


@pytest.fixture()
def sqlite_existing_workspace(ws: workspace.Workspace) -> workspace.Workspace:
    store_strategy = result.StoreStrategy.SQLITE
    with result.Writer(
        ws, result_state_policy=result.ResultStatePolicy.DELETE_BEFORE_WRITE, store_strategy=store_strategy
    ) as writer:
        writer.write(
            identifier="dummy-result-1",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-1"}},
        )

        writer.write(
            identifier="dummy-result-2",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-2"}},
        )

    ws.record_state(timestamp=datetime.datetime.now(), urls=[], store=store_strategy, version=1, distribution_version=1)
    return ws


@pytest.fixture()
def sqlite_existing_workspace_with_partial_results(sqlite_existing_workspace: workspace.Workspace) -> workspace.Workspace:
    shutil.copy2(
        os.path.join(sqlite_existing_workspace.results_path, "results.db"),
        os.path.join(sqlite_existing_workspace.results_path, "results.db.tmp"),
    )
    sqlite_existing_workspace.record_state(
        timestamp=datetime.datetime.now(), urls=[], store=result.StoreStrategy.SQLITE, version=1, distribution_version=1
    )
    assert len(list(sqlite_existing_workspace.state().result_files(sqlite_existing_workspace.path))) == 2
    return sqlite_existing_workspace


@pytest.mark.parametrize(
    "result_state_policy,expected_files,expected_result_count",
    [
        (
            result.ResultStatePolicy.DELETE_BEFORE_WRITE,
            [
                workspace.File(digest="fc882c085f58a6b4", path="results/dummy-result-3.json", algorithm="xxh64"),
                workspace.File(digest="7811995c51f2fdd0", path="results/dummy-result-4.json", algorithm="xxh64"),
            ],
            2,
        ),
        (
            result.ResultStatePolicy.KEEP,
            [
                workspace.File(digest="903bd98818e2382e", path="results/dummy-result-1.json", algorithm="xxh64"),
                workspace.File(digest="19d53931485fa9d0", path="results/dummy-result-2.json", algorithm="xxh64"),
                workspace.File(digest="fc882c085f58a6b4", path="results/dummy-result-3.json", algorithm="xxh64"),
                workspace.File(digest="7811995c51f2fdd0", path="results/dummy-result-4.json", algorithm="xxh64"),
            ],
            4,
        ),
    ],
)
def test_result_writer_flat_file(
    flat_file_existing_workspace: workspace.Workspace,
    result_state_policy: result.ResultStatePolicy,
    expected_files: list[workspace.File],
    expected_result_count: int,
):
    ws = flat_file_existing_workspace
    store_strategy = result.StoreStrategy.FLAT_FILE
    with result.Writer(ws, result_state_policy=result_state_policy, store_strategy=store_strategy) as writer:
        writer.write(
            identifier="dummy-result-3",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-3"}},
        )

        writer.write(
            identifier="dummy-result-4",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-4"}},
        )

        writer.write(
            identifier="dummy-result-4",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-4"}},
        )

    ws.record_state(timestamp=datetime.datetime.now(), urls=[], store=store_strategy, version=1, distribution_version=1)

    state = ws.state()

    assert list(state.result_files(ws.path)) == expected_files
    assert state.result_count(ws.path) == expected_result_count


@pytest.mark.parametrize(
    "result_state_policy,expected_file_count,expected_result_count",
    [
        (
            result.ResultStatePolicy.DELETE_BEFORE_WRITE,
            1,
            2,
        ),
        (
            result.ResultStatePolicy.KEEP,
            1,
            4,
        ),
    ],
)
def test_result_writer_sqlite(
    sqlite_existing_workspace: workspace.Workspace,
    result_state_policy: result.ResultStatePolicy,
    expected_file_count: int,
    expected_result_count: int,
):
    ws = sqlite_existing_workspace
    store_strategy = result.StoreStrategy.SQLITE
    with result.Writer(ws, result_state_policy=result_state_policy, store_strategy=store_strategy) as writer:
        writer.write(
            identifier="dummy-result-3",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-3"}},
        )

        writer.write(
            identifier="dummy-result-4",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-4"}},
        )

        writer.write(
            identifier="dummy-result-4",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-4"}},
        )

    ws.record_state(timestamp=datetime.datetime.now(), urls=[], store=store_strategy, version=1, distribution_version=1)
    state = ws.state()

    # note: since the hash changes on each test run, just confirm the result file count, not the extra metadata
    assert len(list(state.result_files(ws.path))) == expected_file_count
    assert state.result_count(ws.path) == expected_result_count


@pytest.mark.parametrize(
    "result_state_policy,expected_file_count,expected_result_count",
    [
        (
            result.ResultStatePolicy.DELETE_BEFORE_WRITE,
            1,
            2,
        ),
        (
            result.ResultStatePolicy.KEEP,
            1,
            4,
        ),
    ],
)
def test_result_writer_sqlite_with_partial_result(
    sqlite_existing_workspace_with_partial_results: workspace.Workspace,
    result_state_policy: result.ResultStatePolicy,
    expected_file_count: int,
    expected_result_count: int,
):
    ws = sqlite_existing_workspace_with_partial_results
    store_strategy = result.StoreStrategy.SQLITE
    with result.Writer(ws, result_state_policy=result_state_policy, store_strategy=store_strategy) as writer:
        writer.write(
            identifier="dummy-result-3",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-3"}},
        )

        writer.write(
            identifier="dummy-result-4",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-4"}},
        )

        writer.write(
            identifier="dummy-result-4",
            schema=schema.OSSchema(),
            payload={"Vulnerability": {"dummy": "result-4"}},
        )

    ws.record_state(timestamp=datetime.datetime.now(), urls=[], store=store_strategy, version=1, distribution_version=1)
    state = ws.state()

    # note: since the hash changes on each test run, just confirm the result file count, not the extra metadata
    assert len(list(state.result_files(ws.path))) == expected_file_count
    assert state.result_count(ws.path) == expected_result_count



from __future__ import annotations

import json
import os

import pytest
from vunnel import result, workspace
from vunnel.providers.echo import Config, Provider
from vunnel.providers.echo.parser import Parser


class TestParser:
    @pytest.fixture()
    def mock_raw_data(self, helpers):
        """
        Returns stringified version of the following json

        ---
        """
        data = {
            "nginx": {
                "CVE-2021-3618": { "fixed_version": "1.21.0" },
                "CVE-2023-44487": { "fixed_version": "1.25.2" }
            },
            "python": {
                "CVE-2007-4559": { "fixed_version": "3.6.16" },
                "CVE-2009-2940": {},
                "CVE-2009-3720": {},
                "CVE-2013-0340": { "fixed_version": "3.6.15" },
                "CVE-2015-20107": { "fixed_version": "3.7.15" },
                "CVE-2016-3189": { "fixed_version": "3.7.13" },
                "CVE-2018-25032": { "fixed_version": "3.7.14" },
                "CVE-2019-12900": { "fixed_version": "3.7.13" },
                "CVE-2020-10735": { "fixed_version": "3.7.14" },
                "CVE-2020-29396": {},
                "CVE-2021-28861": { "fixed_version": "3.7.14" },
                "CVE-2021-32052": {},
                "CVE-2022-26488": { "fixed_version": "3.7.12" },
                "CVE-2022-37454": { "fixed_version": "3.7.16" },
                "CVE-2022-42919": { "fixed_version": "3.7.15" },
                "CVE-2022-45061": { "fixed_version": "3.7.15" },
                "CVE-2023-24329": { "fixed_version": "3.7.17" },
                "CVE-2023-27043": { "fixed_version": "2.7.18" },
                "CVE-2023-36632": { "fixed_version": "3.11.4" },
                "CVE-2023-40217": { "fixed_version": "3.8.18" },
                "CVE-2024-6232": { "fixed_version": "3.8.20" },
                "CVE-2024-7592": { "fixed_version": "3.8.20" },
                "CVE-2024-9287": { "fixed_version": "3.9.21" }
            },
        }


        return json.dumps(data)

    @pytest.fixture()
    def mock_parsed_data(self):
        """
        Returns the parsed output generated by AlpineDataProvider._load() for the mock_raw_data

        :return:
        """
        release = "rolling"
        data = {
            "nginx": {
                "CVE-2021-3618": { "fixed_version": "1.21.0" },
                "CVE-2023-44487": { "fixed_version": "1.25.2" }
            },
            "python": {
                "CVE-2007-4559": { "fixed_version": "3.6.16" },
                "CVE-2009-2940": {},
                "CVE-2009-3720": {},
                "CVE-2013-0340": { "fixed_version": "3.6.15" },
                "CVE-2015-20107": { "fixed_version": "3.7.15" },
                "CVE-2016-3189": { "fixed_version": "3.7.13" },
                "CVE-2018-25032": { "fixed_version": "3.7.14" },
                "CVE-2019-12900": { "fixed_version": "3.7.13" },
                "CVE-2020-10735": { "fixed_version": "3.7.14" },
                "CVE-2020-29396": {},
                "CVE-2021-28861": { "fixed_version": "3.7.14" },
                "CVE-2021-32052": {},
                "CVE-2022-26488": { "fixed_version": "3.7.12" },
                "CVE-2022-37454": { "fixed_version": "3.7.16" },
                "CVE-2022-42919": { "fixed_version": "3.7.15" },
                "CVE-2022-45061": { "fixed_version": "3.7.15" },
                "CVE-2023-24329": { "fixed_version": "3.7.17" },
                "CVE-2023-27043": { "fixed_version": "2.7.18" },
                "CVE-2023-36632": { "fixed_version": "3.11.4" },
                "CVE-2023-40217": { "fixed_version": "3.8.18" },
                "CVE-2024-6232": { "fixed_version": "3.8.20" },
                "CVE-2024-7592": { "fixed_version": "3.8.20" },
                "CVE-2024-9287": { "fixed_version": "3.9.21" }
            },
        }
        return release, data

    def test_normalize(self, mock_parsed_data, tmpdir):
        p = Parser(
            workspace=workspace.Workspace(tmpdir, "test", create=True),
            url="https://advisory.echohq.com/data.json",
            namespace="echo",
        )
        release = mock_parsed_data[0]
        data = mock_parsed_data[1]

        vuln_records = p._normalize(release, data)
        assert len(vuln_records) > 0
        assert all("Vulnerability" in x for x in vuln_records.values())
        assert sorted(vuln_records.keys()) == sorted(
            [
                "CVE-2007-4559",
                "CVE-2009-2940",
                "CVE-2009-3720",
                "CVE-2013-0340",
                "CVE-2015-20107",
                "CVE-2016-3189",
                "CVE-2018-25032",
                "CVE-2019-12900",
                "CVE-2020-10735",
                "CVE-2020-29396",
                "CVE-2021-28861",
                "CVE-2021-32052",
                "CVE-2021-3618",
                "CVE-2022-26488",
                "CVE-2022-37454",
                "CVE-2022-42919",
                "CVE-2022-45061",
                "CVE-2023-24329",
                "CVE-2023-27043",
                "CVE-2023-36632",
                "CVE-2023-40217",
                "CVE-2023-44487",
                "CVE-2024-6232",
                "CVE-2024-7592",
                "CVE-2024-9287",
            ],
        )


def test_provider_schema(helpers, disable_get_requests, monkeypatch):
    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/input",
    )
    c = Config()
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(root=workspace.root, config=c)
    def mock_download():
        return None

    monkeypatch.setattr(p.parser, "_download", mock_download)

    p.update(None)

    assert workspace.num_result_entries() == 25
    assert workspace.result_schemas_valid(require_entries=True)


def test_provider_via_snapshot(helpers, disable_get_requests, monkeypatch):
    workspace = helpers.provider_workspace_helper(
        name=Provider.name(),
        input_fixture="test-fixtures/input",
    )

    c = Config()
    # keep all of the default values for the result store, but override the strategy
    c.runtime.result_store = result.StoreStrategy.FLAT_FILE
    p = Provider(
        root=workspace.root,
        config=c,
    )

    def mock_download():
        return None

    monkeypatch.setattr(p.parser, "_download", mock_download)

    p.update(None)

    workspace.assert_result_snapshots()



"""
This driver requires a specific Github Personal Access Token (PAT). Generate
one with the necessary scope with:

    https://github.com/settings/tokens/new

The authentication is done over HTTP headers for a PAT. If OAuth apps are
needed, then the authentication header needs to be modified from:

     headers = {"Authorization": "token XYZ123TOKENTOKENTOKEN"}

To:

     headers = {"Authorization": "Bearer XYZ123TOKENTOKENTOKEN"}

"""

from __future__ import annotations

import datetime
import logging
import os
import time
from decimal import Decimal, DecimalException

import requests
from cvss import CVSS3
from cvss.exceptions import CVSS3MalformedError

from vunnel import utils
from vunnel.utils import fdb as db
from vunnel.utils.vulnerability import CVSS, CVSSBaseMetrics

# this is the ecosystem in GHSA to a syft package type
ecosystem_map = {
    "COMPOSER": "composer",
    "GO": "go",
    "MAVEN": "java",
    "NPM": "npm",
    "NUGET": "nuget",
    "PIP": "python",
    "PUB": "dart",
    "RUBYGEMS": "gem",
    "RUST": "rust",
    "SWIFT": "swift",
    "ACTIONS": "github-action",
}

GITHUB_RATE_LIMIT_REMAINING_HEADER = "x-ratelimit-remaining"
GITHUB_RATE_LIMIT_RESET_HEADER = "x-ratelimit-reset"


class Parser:
    def __init__(
        self,
        workspace,
        token,
        download_timeout=125,
        api_url="https://api.github.com/graphql",
        logger=None,
    ):
        self.db = db.connection(workspace.input_path, serializer="json")
        self.download_timeout = download_timeout
        self.api_url = api_url
        self.token = token
        self.timestamp = None
        self.cursor = None
        if not logger:
            logger = logging.getLogger(self.__class__.__name__)
        self.logger = logger

    def _download(self, vuln_cursor=None):
        if not self.token:
            raise ValueError("Github token must be defined")

        """
        Download the advisories from Github via the GraphQL API, using a cursor
        if it was defined in the class. Advisories stay in memory until
        persisted later in the process.
        """
        query = graphql_advisories(timestamp=self.timestamp, cursor=self.cursor, vuln_cursor=vuln_cursor)

        return get_query(self.token, query, self.download_timeout, self.api_url)

    def _parse(self, data):
        """
        Load JSON from `self.json_file`, extract values from interesting fields.
        Sample output from GraphQL request:

            {
              "data": {
                "securityAdvisories": {
                  "nodes": [
                    {
                      "ghsaId": "GHSA-73m2-3pwg-5fgc",
                      "summary": "Critical severity vulnerability that affects waitress",
                      "severity": "CRITICAL",
                      "publishedAt": "2020-02-04T03:07:31Z",
                      "identifiers": [
                        {
                          "type": "GHSA",
                          "value": "GHSA-73m2-3pwg-5fgc"
                        },
                        {
                          "type": "CVE",
                          "value": "CVE-2020-5236"
                        }
                      ],
                      "references": [
                        {
                          "url": "https://github.com/Pylons/waitress/security/advisories/GHSA-73m2-3pwg-5fgc"
                        },
                        {
                          "url": "https://nvd.nist.gov/vuln/detail/CVE-2020-5236"
                        }
                      ],
                      "vulnerabilities": {
                        "pageInfo": {
                          "endCursor": "MQ",
                          "hasNextPage": False
                        },
                        "nodes": [
                          {
                            "package": {
                              "ecosystem": "PIP",
                              "name": "waitress"
                            },
                            "firstPatchedVersion": {
                              "identifier": "1.4.3"
                            }
                          }
                        ]
                      },
                      "description": "### Impact\r\n\r\nWhen waitress[...]"
                    }
                  ]
                }
              }
            }
        """
        advisories = []
        advisory_nodes = data.get("data", {}).get("securityAdvisories", {}).get("nodes", [])
        for node_data in advisory_nodes:
            subquery_cursor = needs_subquery(node_data)
            # An unlikely situation where more than one hundred vulnerabilities
            # are associated with an advisory, and requests need to be done to
            # fetch all the missing ones
            if subquery_cursor:
                ghsaId = node_data["ghsaId"]
                self.logger.info(f"found node that requires a subquery: {ghsaId}")

                extra_vulnerabilities = get_vulnerabilities(
                    self.token,
                    ghsaId,
                    self.timestamp,
                    subquery_cursor,
                    parent_cursor=self.cursor,
                )
                current_vulnerabilities = node_data.get("vulnerabilities", {}).get("nodes", [])
                current_vulnerabilities.extend(extra_vulnerabilities)

            parsed = NodeParser(node_data, logger=self.logger).parse()
            advisories.append(parsed)

        return advisories

    def get(self):
        # determine if a run was completed by looking for a timestamp
        metadata = self.db.get_metadata()

        # why rstrip(Z)? Previous code had been incorrectly adding a Z to the end of the timestamp manually
        # instead of using the isoformat() method. This lead to a parsing problem with the github API (naturally).
        # in the future this rstrip() should be removed when all cached data in CI is updated.
        self.timestamp = metadata.data.get("timestamp")
        if self.timestamp:
            self.timestamp = self.timestamp.rstrip("Z")

        current_timestamp = datetime.datetime.now(tz=datetime.UTC).isoformat()
        has_cursor = True

        # Process everything that was persisted first
        for node_data in self.db.get_all():
            yield NodeParser(node_data.load(), logger=self.logger).parse()

        while has_cursor:
            # download graphql data as json, if the timestamp is present, then the
            # download will retrieve advisories that have been updated from that
            # point in time, if a cursor is present, it will be used as well
            data = self._download()

            if "errors" in data:
                raise RuntimeError(f"Error downloading advisories: {data['errors']}")

            page_info = data["data"]["securityAdvisories"]["pageInfo"]
            if page_info["hasNextPage"]:
                self.cursor = has_cursor = page_info.get("endCursor")
            else:
                has_cursor = False

            advisories = self._parse(data)
            for advisory in advisories:
                record = self.db.create(advisory.data["ghsaId"])
                record.commit(advisory.data)
                yield advisory

        # No more work to do. Persist the timestamp and return
        metadata.data["timestamp"] = current_timestamp
        metadata.commit()


@utils.retry_with_backoff()
def get_query(token, query, timeout=125, api_url="https://api.github.com/graphql"):
    logger = logging.getLogger("get-query")

    headers = {"Authorization": f"token {token}"}
    logger.info(f"downloading github advisories from {api_url}")

    response = requests.post(api_url, json={"query": query}, timeout=timeout, headers=headers)
    if GITHUB_RATE_LIMIT_REMAINING_HEADER in response.headers and GITHUB_RATE_LIMIT_RESET_HEADER in response.headers:
        remaining = int(response.headers[GITHUB_RATE_LIMIT_REMAINING_HEADER])
        # reset time is the time in UNIX Epoch Seconds at which
        # the rate limit will reset.
        reset_time = int(response.headers[GITHUB_RATE_LIMIT_RESET_HEADER])
        logger.debug(f"github rate limit has {remaining} requests left {reset_time}")
        if remaining < 10:
            current_time = int(time.time())
            sleep_time = reset_time - current_time
            # note that the rate limit resets 1x / hour, so this could be a long time
            if sleep_time > 1 and sleep_time < 3600:  # never sleep for more than 1 hour
                logger.info(f"sleeping for {sleep_time} seconds to allow GitHub rate limit to reset")
                time.sleep(sleep_time)
            elif sleep_time > 3600:
                raise Exception(
                    f"github rate limit exhaused and not expected to reset for {sleep_time} seconds. Try again later.",
                )
    response.raise_for_status()
    if response.status_code == 200:
        return response.json()
    raise Exception(f"Unable to retrieve Github Advisories. HTTP Code: {response.status_code}")


def get_advisory(ghsaId, data):
    """
    Given a list of advisories, find the one associated with the ghsaId. The
    data is expected as the raw JSON response from the GraphQL, so processing
    needs to traverse into the nodes
    """
    try:
        nodes = data["data"]["securityAdvisories"]["nodes"]
    except KeyError:
        return {}
    for node in nodes:
        if node.get("ghsaId", "") == ghsaId:
            return node
    return {}


def get_vulnerabilities(token, ghsaId, timestamp, vuln_cursor, parent_cursor):
    """
    In the improbable case that an Advisory is associated with more than 100
    (Github's GraphQL limit) these will need to get fetched until the cursor is
    exhausted.
    This function is only executed if it has been determined that pagination is
    present from the securityAdvisories JSON response::

      {
        ...
            "vulnerabilities": {
              "pageInfo": {
                "hasNextPage": false,
                "endCursor": "Mg"
              },
              "nodes": [
                {
                  "package": {
                    "ecosystem": "NPM",
                    "name": "event-stream"
                  },
                  "firstPatchedVersion": {
                    "identifier": "4.0.0"
                  }
                }
              ]
            }
        ...
      }


    """
    logger = logging.getLogger("get-vulnerabilities")

    nodes = []
    while vuln_cursor is not None:
        logger.info(f"fetching extra vulnerability from {ghsaId}, page count: {len(nodes) + 1}")

        query = graphql_advisories(parent_cursor, timestamp, vuln_cursor)
        data = get_query(token, query)
        advisory = get_advisory(ghsaId, data)

        # data will retrieve the 100 advisories that the parent cursor
        # initially got, we must find the specific advisory needed for
        # pagination using the ghsaId
        vulnerabilities = advisory.get("vulnerabilities", {})
        page_info = vulnerabilities.get("pageInfo", {})
        vuln_cursor = page_info.get("endCursor") if page_info.get("hasNextPage") else None

        for vulnerability in vulnerabilities.get("nodes", []):
            nodes.append(vulnerability)

    return nodes


def needs_subquery(data):
    """
    This will probably never require a subquery, but still check if it
    needs to, set the boolean.

    Checks for `"hasNextPage"` within the vulnerabilities::

      "vulnerabilities": {
        "pageInfo": {
          "endCursor": "MQ",
          "hasNextPage": true
        },
        "nodes": [
          {
            "package": {
              "ecosystem": "NPM",
              "name": "event-stream"
            },
            "firstPatchedVersion": {
              "identifier": "4.0.0"
            }
          }
        ]
      }

    This function is not part of the parser because it does not alter or
    extract any useful information needed for processing, preventing double
    parsing.
    """
    pageInfo = data.get("vulnerabilities", {}).get("pageInfo", {})
    if pageInfo.get("hasNextPage", True):
        return pageInfo.get("endCursor")
    return False


def graphql_advisories(cursor=None, timestamp=None, vuln_cursor=None):
    """
    The cursor needs to be the `endCursor` for the last successful query. The
    feed will go fetch from the beginning of all the feeds available, in
    chunks of 100 which is the limit per request. For the roughly 1700
    advisories this would mean less than 20 requests.

    There are four distinct GraphQL requests:

    * No cursor, and no timestamp: the first request ever to the advisory database
    * A cursor, but no timestamp: The first batch of requests ever after the
      initial request to the database
    * No cursor with a timestamp: A first batch was completed, and this is the
      first request for a new iteration. Timestamp is used to retrieve any
      updates since the last batch was finished
    * A cursor with timestamp: A first batch was completed, and this follows
      after the first request is done which provides the cursor.

    Example GraphQL query (use: https://developer.github.com/v4/explorer/) ::

        {
          securityAdvisories(
            classifications: [GENERAL, MALWARE]
            orderBy: {field: PUBLISHED_AT, direction: ASC}
            first: 10
          ) {
            nodes {
              ghsaId
              classification
              summary
              severity
              cvss {
                score
                vectorString
              }
              identifiers {
                type
                value
              }
              references {
                url
              }
              vulnerabilities(
                classifications: [GENERAL, MALWARE]
                first: 100
                orderBy: {field: UPDATED_AT, direction: ASC}
              ) {
                pageInfo {
                  endCursor
                  hasNextPage
                }
                nodes {
                  package {
                    ecosystem
                    name
                  }
                  firstPatchedVersion {
                    identifier
                  }
                  vulnerableVersionRange
                }
              }
              publishedAt
              updatedAt
              withdrawnAt
            }
            pageInfo {
              endCursor
              hasNextPage
              hasPreviousPage
              startCursor
            }
          }
        }

    If trying to get a single GHSA, the `securityAdvisories` field needs to be updated
    with and identifier, to::

      securityAdvisories(
        classifications: [GENERAL, MALWARE]
        orderBy: {field: PUBLISHED_AT
        direction: ASC}, first: 10
        identifier: {type: GHSA, value: "GHSA-pp7h-53gx-mx7r"}
      ) {
    """
    query_func = "securityAdvisories(orderBy: {field: %s, direction: ASC}, "
    updatedSince = ""
    after = ""
    vuln_after = ""
    if timestamp:
        query_func = query_func % "UPDATED_AT"
        updatedSince = f'updatedSince: "{timestamp}", '
    else:
        query_func = query_func % "PUBLISHED_AT"

    if cursor:
        after = f'after: "{cursor}", '

    caller = f"{query_func}{after}{updatedSince}classifications: [GENERAL, MALWARE], first: 100)"

    if vuln_cursor:
        vuln_after = f'after: "{vuln_cursor}", '
    vulnerabilities = f"{vuln_after}classifications: [GENERAL, MALWARE], first: 100, orderBy: {{field: UPDATED_AT, direction: ASC}}"

    return f"""
    {{
      {caller} {{
        nodes {{
          ghsaId
          classification
          summary
          severity
          cvss {{
            score
            vectorString
          }}
          identifiers {{
            type
            value
          }}
          references {{
            url
          }}
          vulnerabilities({vulnerabilities}) {{
            pageInfo {{
              endCursor
              hasNextPage
            }}
            nodes {{
              package {{
                ecosystem
                name
              }}
              firstPatchedVersion {{
                identifier
              }}
              vulnerableVersionRange
            }}
          }}
          publishedAt
          updatedAt
          withdrawnAt
        }}
        pageInfo {{
          endCursor
          hasNextPage
          hasPreviousPage
          startCursor
        }}
      }}
    }}
    """


class NodeParser(dict):
    __parsers__ = (
        "_classification",
        "_severity",
        "_cvss",
        "_fixedin",
        "_summary",
        "_url",
        "_cves",
        "_published",
        "_updated",
        "_withdrawn",
    )

    def __init__(self, data, logger=None):
        self.description = None
        self.identifier = None
        self.summary = None
        self.cves = []
        self.data = data
        self.ecosystems = set()
        if not logger:
            logger = logging.getLogger(self.__class__.__name__)
        self.logger = logger

    def parse(self):
        for parser in self.__parsers__:
            getattr(self, parser)()
        return self

    def __getattr__(self, attr):
        if attr in self:
            return self[attr]
        raise AttributeError(f"No such attribute: {attr}")

    def _classification(self):
        classification = self.data.get("classification", "GENERAL")
        self["Classification"] = classification

    def _severity(self):
        """
        Severity is extracted and expected to be upper-case:

            "severity": "CRITICAL",

        This is transformed to meet the schema on the consumer to:

            "Critical"

        All severities from Github are::

            'CRITICAL'
            'HIGH'
            'LOW'
            'MODERATE'


        """
        severity_map = {
            "LOW": "Low",
            "MODERATE": "Medium",
            "HIGH": "High",
            "CRITICAL": "Critical",
        }
        severity = self.data.get("severity")
        self["Severity"] = severity_map.get(severity, "Unknown")

    def _make_cvss(self, cvss_vector: str, vulnerability_id: str) -> CVSS | None:
        try:
            cvss_vector = cvss_vector.removesuffix("/")
            cvss3_obj = CVSS3(cvss_vector)

            cvss_object = CVSS(
                version=f"3.{cvss3_obj.minor_version}",
                vector_string=cvss_vector,
                base_metrics=CVSSBaseMetrics(
                    base_score=float(cvss3_obj.base_score.quantize(Decimal("0.1"))),
                    exploitability_score=float(cvss3_obj.esc.quantize(Decimal("0.1"))),
                    impact_score=float(cvss3_obj.isc.quantize(Decimal("0.1"))),
                    base_severity=cvss3_obj.severities()[0],
                ),
                status="N/A",
            )
        except (CVSS3MalformedError, DecimalException, AttributeError):
            self.logger.exception(
                "error transforming CVSS vector %s, skipping it for %s",
                cvss_vector,
                vulnerability_id,
            )
            cvss_object = None

        return cvss_object

    def _cvss(self):
        cvss = self.data.get("cvss")

        if cvss:
            vector = cvss.get("vectorString")

            if vector:
                self["CVSS"] = self._make_cvss(vector, self.data.get("ghsaId"))

    def _fixedin(self):
        """
        Extracts `identifier` key from `firstPatchedVersion':

            "vulnerabilities": {
              "nodes": [
                {
                  "package": {
                    "ecosystem": "PIP",
                    "name": "waitress"
                  },
                  "firstPatchedVersion": {
                    "identifier": "1.4.3"
                  },
                  "vulnerableVersionRange": ">= 1.2.0, < 1.4.3",
                }
              ]
            }
        """
        self["FixedIn"] = []
        vulnerabilities = self.data.get("vulnerabilities", {}).get("nodes", [])
        for item in vulnerabilities:
            # identify the type of package, if this is a DEB or RPM or
            # something we haven't seen before, ignore it
            github_ecosystem = item.get("package", {}).get("ecosystem")
            ecosystem = ecosystem_map.get(github_ecosystem)
            if ecosystem:
                self.ecosystems.add(ecosystem)

                try:
                    identifier = item.get("firstPatchedVersion", {}).get("identifier", "None")
                except AttributeError:
                    identifier = "None"
                package_name = item.get("package", {}).get("name")

                version_range = item.get("vulnerableVersionRange", "").replace(",", "")
                self["FixedIn"].append(
                    {
                        "name": package_name,
                        "identifier": identifier,
                        "ecosystem": ecosystem,
                        "namespace": f"github:{ecosystem}",
                        "range": version_range,
                    },
                )
            else:
                # Log vuln skipped for unknown ecosystem
                self.logger.debug("dropping github vuln from unmapped ecosystem: %s", github_ecosystem)

    def _summary(self):
        self["Summary"] = self.data.get("summary")

    def _published(self):
        self["published"] = self.data.get("publishedAt")

    def _updated(self):
        self["updated"] = self.data.get("updatedAt")

    def _withdrawn(self):
        self["withdrawn"] = self.data.get("withdrawnAt")

    def _url(self):
        """
        This creates the advisory URL which doesn't exist as part of the API,
        using the ID.
        """
        self["url"] = os.path.join("https://github.com/advisories/", self.data.get("ghsaId"))

    def _cves(self):
        cves = []
        for identifier in self.data.get("identifiers", []):
            value = identifier["value"]
            if value.startswith("GHSA"):
                continue
            cves.append(value)
        self["CVE"] = cves
        self["Metadata"] = {"CVE": cves}
        self["ghsaId"] = self.data.get("ghsaId")



from __future__ import annotations

import pytest
from vunnel.utils import fdb as db


class TestGetJSON:
    @pytest.mark.parametrize("name", ["doesnotexist", "doesnotexist.json"])
    def test_does_not_find_file_name(self, name, tmpdir):
        conn = db.connection(tmpdir.strpath, serializer="json")
        assert conn.get(name) is None

    @pytest.mark.parametrize("name", ["exists", "exists.json"])
    def test_finds_file_name(self, name, tmpdir):
        _file = tmpdir.join("exists.json")
        _file.write("{}")
        conn = db.connection(tmpdir.strpath, serializer="json")
        record = conn.get(name)
        record.load()
        assert record.data == {}

    def test_ignores_other_ext(self, tmpdir):
        _file = tmpdir.join("exists.vim")
        _file.write("{}")
        conn = db.connection(tmpdir.strpath, serializer="json")
        record = conn.get("exists")
        assert record is None

    @pytest.mark.parametrize("name", ["new", "new.json"])
    def test_creates_data(self, name, tmpdir):
        conn = db.connection(tmpdir.strpath, serializer="json")
        record = conn.create(name)
        record.commit({"a": 1})
        record = conn.get(name)
        record.load()
        assert record.data == {"a": 1}

    def test_gets_all_files(self, tmpdir):
        conn = db.connection(tmpdir.strpath, serializer="json")
        for i in range(9):
            record = conn.create(str(i))
            record.commit({"name": str(i)})
        records = conn.get_all()
        assert len(list(records)) == 9


class TestMeta:
    def test_doesnt_get_mixed(self, tmpdir):
        conn = db.connection(tmpdir.strpath, serializer="json")
        for i in range(9):
            record = conn.create(str(i))
            record.commit({})
        meta = conn.get_metadata()
        meta.commit({"name": "metadata"})
        meta.commit({"type": "json"})
        reloaded_meta = conn.get_metadata()
        reloaded_meta.load()
        assert reloaded_meta.data == {"type": "json"}
        records = conn.get_all()
        assert len(list(records)) == 9

    def test_updates(self, tmpdir):
        conn = db.connection(tmpdir.strpath, serializer="json")
        meta = conn.get_metadata()
        meta.commit({"name": "metadata"})
        meta.data["type"] = "json"
        meta.commit()

        reloaded_meta = conn.get_metadata()
        reloaded_meta.load()
        assert reloaded_meta.data["type"] == "json"
        assert reloaded_meta.data["name"] == "metadata"


class TestGetRaw:
    @pytest.mark.parametrize("name", ["doesnotexist", "doesnotexist.txt"])
    def test_does_not_find_file_name(self, name, tmpdir):
        conn = db.connection(tmpdir.strpath, serializer="raw")
        assert conn.get(name) is None

    @pytest.mark.parametrize("name", ["exists", "exists.txt"])
    def test_finds_file_name(self, name, tmpdir):
        _file = tmpdir.join("exists.txt")
        _file.write("Some Text Here")
        conn = db.connection(tmpdir.strpath, serializer="raw")
        record = conn.get(name)
        record.load()
        assert record.data == "Some Text Here"

    @pytest.mark.parametrize("name", ["new", "new.txt"])
    def test_creates_data(self, name, tmpdir):
        conn = db.connection(tmpdir.strpath, serializer="raw")
        record = conn.create(name)
        record.commit("new text")
        record = conn.get(name)
        record.load()
        assert record.data == "new text"
